from pathlib import Path
import re
import stat

import rpm
from rpmlint.checks.AbstractCheck import AbstractCheck


class TmpFilesCheck(AbstractCheck):
    """
    Validate that temporary files meet tmpfiles.d packaging rules.
    """

    # interesting types in tmpfiles.d configuration file (see tmpfiles.d(5))
    interesting_types = ('f', 'F', 'w', 'd', 'D', 'p', 'L', 'c', 'b')

    def check(self, pkg):
        if pkg.is_source:
            return

        for fname, pkgfile in pkg.files.items():
            if not fname.startswith('/usr/lib/tmpfiles.d/'):
                continue
            if not stat.S_ISREG(pkgfile.mode):
                self.output.add_info('W', pkg, 'tmpfile-not-regular-file',
                                     fname)
                continue

            if pkgfile.is_ghost:
                continue

            self._check_pre_tmpfile(fname, pkg)
            self._check_post_tmpfile(fname, pkg)
            self._check_tmpfile_in_filelist(pkgfile, pkg)

    def _check_pre_tmpfile(self, fname, pkg):
        """
        Check if the %pre section doesn't contain 'systemd-tmpfiles --create'
        call.

        Print a warning if there is systemd-tmpfiles call in the %pre section.
        """
        pre = pkg[rpm.RPMTAG_PREIN]

        basename = Path(fname).name
        tmpfiles_regex = re.compile(r'systemd-tmpfiles --create .*%s'
                                    % re.escape(basename))

        if pre and tmpfiles_regex.search(pre):
            self.output.add_info('W', pkg, 'pre-with-tmpfile-creation', fname)

    def _check_post_tmpfile(self, fname, pkg):
        """
        Check if the %post section contains 'systemd-tmpfiles --create' call.

        Print a warning if there is no such call in the %post section.
        """
        post = pkg[rpm.RPMTAG_POSTIN]

        basename = Path(fname).name
        tmpfiles_regex = re.compile(r'systemd-tmpfiles --create .*%s'
                                    % re.escape(basename))

        if post and tmpfiles_regex.search(post):
            return
        self.output.add_info('W', pkg, 'post-without-tmpfile-creation', fname)

    def _check_tmpfile_in_filelist(self, pkgfile, pkg):
        """
        Check if the tmpfile is listed in the filelist and marked as %ghost.

        Please note that a tmpfile that doesn't exist during the build can't
        be in the filelist without %ghost directive otherwise rpm wouldn't
        build it.

        Print a 'tmpfile-not-in-filelist' warning while it's not in the
        filelist (and therefore not marked as %ghost).
        """
        with open(pkgfile.path) as inputf:
            for line in inputf:
                # skip comments
                line = line.split('#')[0].split('\n')[0]
                line = line.lstrip()
                if not len(line):
                    continue

                # the format is:
                # Type Path Mode UID  GID  Age Argument
                line = re.split(r'\s+', line)
                if len(line) < 3:
                    continue
                # we only need Type and Path
                tmpfiles_type = line[0]
                tmpfiles_path = line[1]
                if tmpfiles_type.endswith('!'):
                    tmpfiles_type = tmpfiles_type[:-1]
                if tmpfiles_type not in self.interesting_types:
                    continue

                if tmpfiles_path not in pkg.files:
                    self.output.add_info('W', pkg, 'tmpfile-not-in-filelist',
                                         tmpfiles_path)



from mockdata.mock_build_root import BuildrootPackage
import pytest
from rpmlint.checks.BuildRootAndDateCheck import BuildRootAndDateCheck
from rpmlint.filter import Filter

from Testing import CONFIG


@pytest.fixture(scope='function', autouse=True)
def buildrootcheck():
    CONFIG.info = True
    output = Filter(CONFIG)
    test = BuildRootAndDateCheck(CONFIG, output)
    return output, test


@pytest.mark.parametrize('package', [BuildrootPackage])
def test_build_root(package, buildrootcheck):
    output, test = buildrootcheck
    test.prepare_regex('/home/marxin/rpmbuild/BUILDROOT/%{NAME}-%{VERSION}-%{RELEASE}.x86_64')
    test.check(package)
    out = output.print_results(output.results)
    assert 'E: file-contains-buildroot /bin/trace' in out



import argparse
from pathlib import Path
import sys

from rpmlint.helpers import print_warning
from rpmlint.lint import Lint
from rpmlint.rpmdiff import Rpmdiff
from rpmlint.version import __version__


__copyright__ = """
    Copyright (C) 2006 Mandriva
    Copyright (C) 2009 Red Hat, Inc.
    Copyright (C) 2009 Ville Skyttä
    Copyright (C) 2017 SUSE LINUX GmbH
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""


def process_diff_args(argv):
    """
    Process the passed arguments and return the result
    :param argv: passed arguments
    """

    parser = argparse.ArgumentParser(prog='rpmdiff',
                                     description='Shows basic differences between two rpm packages',
                                     epilog="""When using the -i or -e options,
                                               separate values from package arguments with '--',
                                               e.g.: 'rpmdiff -i 5 T -- old.rpm new.rpm' or place
                                               the options _after_ the package arguments.""")
    parser.add_argument('old_package', metavar='RPM_ORIG', type=Path, help='the old package')
    parser.add_argument('new_package', metavar='RPM_NEW', type=Path, help='the new package')
    parser.add_argument('-V', '--version', action='version', version=__version__, help='show package version and exit')
    parser.add_argument('-i', '--ignore', nargs='+', default=None, choices=['S', 'M', '5', 'D', 'N', 'L', 'V', 'U', 'G', 'F', 'T'],
                        help="""file property to ignore when calculating differences.
                                Valid values are: S (size), M (mode), 5 (checksum), D (device),
                                N (inode), L (number of links), V (vflags), U (user), G (group),
                                F (digest), T (time)""")
    parser.add_argument('-e', '--exclude', metavar='GLOB', nargs='+', default=None,
                        help="""Paths to exclude when showing differences.
                                Takes a glob. When absolute (starting with /)
                                all files in a matching directory are excluded as well.
                                When relative, files matching the pattern anywhere
                                are excluded but not directory contents.""")

    options = parser.parse_args(args=argv)

    # convert options to dict
    options_dict = vars(options)
    return options_dict


def process_lint_args(argv):
    """
    Process the passed arguments and return the result
    :param argv: passed arguments
    """

    parser = argparse.ArgumentParser(prog='rpmlint',
                                     description='Check for common problems in rpm packages')
    parser.add_argument('rpmfile', nargs='*', type=Path, help='files to be validated by rpmlint')
    parser.add_argument('-V', '--version', action='version', version=__version__, help='show package version and exit')
    parser.add_argument('-c', '--config', type=_validate_conf_location, help='load up additional configuration data from specified path (file or directory with *.toml files)')
    parser.add_argument('-e', '--explain', nargs='+', default='', help='provide detailed explanation for one specific message id')
    parser.add_argument('-r', '--rpmlintrc', '--file', type=_is_file_path, help='load up specified rpmlintrc file')
    parser.add_argument('-v', '--verbose', '--info', action='store_true', help='provide detailed explanations where available')
    parser.add_argument('-p', '--print-config', action='store_true', help='print the settings that are in effect when using the rpmlint')
    parser.add_argument('-i', '--installed', nargs='+', default='', help='installed packages to be validated by rpmlint')
    parser.add_argument('-t', '--time-report', action='store_true', help='print time report for run checks')
    parser.add_argument('-T', '--profile', action='store_true', help='print cProfile report')
    parser.add_argument('--ignore-unused-rpmlintrc', action='store_true',
                        help='Do not report "unused-rpmlintrc-filter" errors')
    parser.add_argument('--checks',
                        help='Debugging option that enables only selected checks (separated by comma)')
    lint_modes_parser = parser.add_mutually_exclusive_group()
    lint_modes_parser.add_argument('-s', '--strict', action='store_true', help='treat all messages as errors')
    lint_modes_parser.add_argument('-P', '--permissive', action='store_true', help='treat individual errors as non-fatal')

    # print help if there is no argument
    if len(argv) < 1:
        parser.print_help()
        sys.exit(0)

    options = parser.parse_args(args=argv)

    # make sure rpmlintrc exists
    if options.rpmlintrc and not options.rpmlintrc.exists():
        print_warning(f"User specified rpmlintrc '{options.rpmlintrc}' does not exist")
        exit(2)
    # validate all the rpmlfile options to be either file or folder
    f_path = set()
    invalid_path = False
    for item in options.rpmfile:
        p_path = Path()
        pattern = None
        for pos, component in enumerate(item.parts):
            if ('*' in component) or ('?' in component):
                pattern = '/'.join(item.parts[pos:])
                break
            p_path = p_path / component
        p_path = list(p_path.glob(pattern)) if pattern else [p_path]

        for path in p_path:
            if not path.exists():
                print_warning(f"The file or directory '{path}' does not exist")
                invalid_path = True
        f_path.update(p_path)

    if invalid_path:
        exit(2)
    # convert options to dict
    options_dict = vars(options)
    # use computed rpmfile
    options_dict['rpmfile'] = list(f_path)
    return options_dict


def _validate_conf_location(string):
    """
    Help validate configuration location during argument parsing.

    We accept either one configuration file or a directory (then it processes
    all *.toml files in this directory). It exits the program if location
    doesn't exist.

    Args:
        string: A string representing configuration path (file or directory).

    Returns:
        A list with individual paths for each configuration file found.
    """
    config_paths = []
    path = Path(string)

    # Exit if file or dir doesn't exist
    if not path.exists():
        print_warning(
            f"File or dir with user specified configuration '{string}' does not exist")
        exit(2)

    if path.is_dir():
        config_paths.extend(path.glob('*.toml'))
    elif path.is_file():
        config_paths.append(path)

    return config_paths


def _is_file_path(path):
    p = Path(path)
    if not p.is_file():
        raise argparse.ArgumentTypeError(f'{path} is not a valid file path')
    return p


def lint():
    """
    Main wrapper for lint command processing
    """
    options = process_lint_args(sys.argv[1:])

    lint = Lint(options)
    sys.exit(lint.run())


def diff():
    """
    Main wrapper for diff command processing
    """
    options = process_diff_args(sys.argv[1:])
    d = Rpmdiff(options['old_package'], options['new_package'],
                ignore=options['ignore'], exclude=options['exclude'])
    textdiff = d.textdiff()
    if textdiff:
        print(textdiff)
    sys.exit(int(d.differs()))



#############################################################################
# Project         : Mandriva Linux
# Module          : rpmlint
# File            : MenuCheck.py
# Author          : Frederic Lepied
# Created On      : Mon Mar 20 07:43:37 2000
#############################################################################

import re
import stat
import subprocess

import rpm
from rpmlint.checks.AbstractCheck import AbstractCheck
from rpmlint.helpers import ENGLISH_ENVIRONMENT


menu_file_regex = re.compile(r'^/usr/lib/menu/([^/]+)$')
old_menu_file_regex = re.compile(r'^/usr/share/(gnome/apps|applnk)/([^/]+)$')
package_regex = re.compile(r'\?package\((.*)\):')
needs_regex = re.compile(r'needs=(\"([^\"]+)\"|([^ %s\"]+))' % '\t')
section_regex = re.compile(r'section=(\"([^\"]+)\"|([^ %s\"]+))' % '\t')
title_regex = re.compile(r'[\"\s]title=(\"([^\"]+)\"|([^ %s\"]+))' % '\t')
longtitle_regex = re.compile(r'longtitle=(\"([^\"]+)\"|([^ %s\"]+))' % '\t')
command_regex = re.compile(r'command=(?:\"([^\"]+)\"|([^ %s\"]+))' % '\t')
icon_regex = re.compile(r'icon=\"?([^\" ]+)')
update_menus_regex = re.compile(r'^[^#]*update-menus', re.MULTILINE)
xpm_ext_regex = re.compile(r'/usr/share/icons/(mini/|large/).*\.xpm$')
version_regex = re.compile(r'([0-9.][0-9.]+)($|\s)')
xdg_migrated_regex = re.compile(r'xdg=\"?([^\" ]+)')


class MenuCheck(AbstractCheck):
    def __init__(self, config, output):
        super().__init__(config, output)
        self.valid_sections = self.config.configuration['ValidMenuSections']
        self.standard_needs = self.config.configuration['ExtraMenuNeeds']
        self.icon_paths = self.config.configuration['IconPath']
        self.launchers = self.config.configuration['MenuLaunchers']
        self.icon_ext_regex = re.compile(self.config.configuration['IconFilename'])
        # compile regexps
        for value in self.launchers.values():
            value['regexp'] = re.compile(value['regexp'])

    def check_binary(self, pkg):
        files = pkg.files
        menus = []

        for fname, pkgfile in files.items():
            # Check menu files
            res = menu_file_regex.search(fname)
            mode = pkgfile.mode
            if res:
                basename = res.group(1)
                if not stat.S_ISREG(mode):
                    self.output.add_info('E', pkg, 'non-file-in-menu-dir', fname)
                else:
                    if basename != pkg.name:
                        self.output.add_info('W', pkg, 'non-coherent-menu-filename', fname)
                    if mode & 0o444 != 0o444:
                        self.output.add_info('E', pkg, 'non-readable-menu-file', fname)
                    if mode & 0o111:
                        self.output.add_info('E', pkg, 'executable-menu-file', fname)
                    menus.append(fname)
            else:
                # Check old menus from KDE and GNOME
                res = old_menu_file_regex.search(fname)
                if res:
                    if stat.S_ISREG(mode):
                        self.output.add_info('E', pkg, 'old-menu-entry', fname)
                else:
                    # Check non transparent xpm files
                    res = xpm_ext_regex.search(fname)
                    if res and stat.S_ISREG(mode) and not pkg.grep('None",', fname):
                        self.output.add_info('W', pkg, 'non-transparent-xpm', fname)
                if fname.startswith('/usr/lib64/menu'):
                    self.output.add_info('E', pkg, 'menu-in-wrong-dir', fname)

        if menus:
            postin = pkg[rpm.RPMTAG_POSTIN] or \
                pkg.scriptprog(rpm.RPMTAG_POSTINPROG)
            if not postin:
                self.output.add_info('E', pkg, 'menu-without-postin')
            elif not update_menus_regex.search(postin):
                self.output.add_info('E', pkg, 'postin-without-update-menus')

            postun = pkg[rpm.RPMTAG_POSTUN] or \
                pkg.scriptprog(rpm.RPMTAG_POSTUNPROG)
            if not postun:
                self.output.add_info('E', pkg, 'menu-without-postun')
            elif not update_menus_regex.search(postun):
                self.output.add_info('E', pkg, 'postun-without-update-menus')

            directory = pkg.dir_name()
            for f in menus:
                # remove comments and handle cpp continuation lines
                text = subprocess.run(('/lib/cpp', directory + f), stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env=ENGLISH_ENVIRONMENT, text=True).stdout
                if text.endswith('\n'):
                    text = text[:-1]

                for line in text.splitlines():
                    if not line.startswith('?'):
                        continue
                    res = package_regex.search(line)
                    if res:
                        package = res.group(1)
                        if package != pkg.name:
                            self.output.add_info('W', pkg,
                                                 'incoherent-package-value-in-menu',
                                                 package, f)
                    else:
                        self.output.add_info('I', pkg, 'unable-to-parse-menu-entry', line)

                    command = True
                    res = command_regex.search(line)
                    if res:
                        command_line = (res.group(1) or res.group(2)).split()
                        command = command_line[0]
                        for launcher in self.launchers.values():
                            if not launcher['regexp'].search(command):
                                continue
                            found = False
                            if launcher['binaries']:
                                found = '/bin/' + command_line[0] in files or \
                                    '/usr/bin/' + command_line[0] in files or \
                                    '/usr/X11R6/bin/' + command_line[0] \
                                    in files
                                if not found:
                                    for i in launcher['binaries']:
                                        if i in pkg.req_names:
                                            found = True
                                            break
                                if not found:
                                    self.output.add_info('E', pkg,
                                                         'use-of-launcher-in-menu-but-no-requires-on',
                                                         launcher['binaries'][0])
                            command = command_line[1]
                            break

                        if command[0] == '/':
                            if command not in files:
                                self.output.add_info('W',
                                                     pkg, 'menu-command-not-in-package',
                                                     command)
                        elif not ('/bin/' + command in files or
                                  '/usr/bin/' + command in files or
                                  '/usr/X11R6/bin/' + command in files):
                            self.output.add_info('W', pkg, 'menu-command-not-in-package',
                                                 command)
                    else:
                        self.output.add_info('W', pkg, 'missing-menu-command')
                        command = False

                    res = longtitle_regex.search(line)
                    if res:
                        grp = res.groups()
                        title = grp[1] or grp[2]
                        if title[0] != title[0].upper():
                            self.output.add_info('W', pkg, 'menu-longtitle-not-capitalized',
                                                 title)
                        res = version_regex.search(title)
                        if res:
                            self.output.add_info('W', pkg, 'version-in-menu-longtitle',
                                                 title)
                    else:
                        self.output.add_info('E', pkg, 'no-longtitle-in-menu', f)
                        title = None

                    res = title_regex.search(line)
                    if res:
                        grp = res.groups()
                        title = grp[1] or grp[2]
                        if title[0] != title[0].upper():
                            self.output.add_info('W', pkg, 'menu-title-not-capitalized',
                                                 title)
                        res = version_regex.search(title)
                        if res:
                            self.output.add_info('W', pkg, 'version-in-menu-title', title)
                        if '/' in title:
                            self.output.add_info('E', pkg, 'invalid-title', title)
                    else:
                        self.output.add_info('E', pkg, 'no-title-in-menu', f)
                        title = None

                    res = needs_regex.search(line)
                    if res:
                        grp = res.groups()
                        needs = (grp[1] or grp[2]).lower()
                        if needs in ('x11', 'text', 'wm'):
                            res = section_regex.search(line)
                            if res:
                                grp = res.groups()
                                section = grp[1] or grp[2]
                                # don't warn entries for sections
                                if command and section not in self.valid_sections:
                                    self.output.add_info('E', pkg, 'invalid-menu-section',
                                                         section, f)
                            else:
                                self.output.add_info('I', pkg, 'unable-to-parse-menu-section',
                                                     line)
                        elif needs not in self.standard_needs:
                            self.output.add_info('I', pkg, 'strange-needs', needs, f)
                    else:
                        self.output.add_info('I', pkg, 'unable-to-parse-menu-needs', line)

                    res = icon_regex.search(line)
                    if res:
                        icon = res.group(1)
                        if not self.icon_ext_regex.search(icon):
                            self.output.add_info('W', pkg, 'invalid-menu-icon-type', icon)
                        if icon[0] == '/' and needs == 'x11':
                            self.output.add_info('W', pkg, 'hardcoded-path-in-menu-icon',
                                                 icon)
                        else:
                            for value in self.icon_paths.values():
                                if (value['path'] + icon) not in files:
                                    self.output.add_info('E',
                                                         pkg, value['type'] + '-icon-not-in-package',
                                                         icon, f)
                    else:
                        self.output.add_info('W', pkg, 'no-icon-in-menu', title)

                    res = xdg_migrated_regex.search(line)
                    if res:
                        if res.group(1).lower() != 'true':
                            self.output.add_info('E', pkg, 'non-xdg-migrated-menu')
                    else:
                        self.output.add_info('E', pkg, 'non-xdg-migrated-menu')



import rpm

from Testing import get_tested_mock_package


UnexpandedMacroPackage = get_tested_mock_package(
    lazyload=True,
    name='unexpanded1',
    header={
        'requires': [],
        'provides': ['/%notreally', 'unexpanded1 = 0-0'],
        'suggests': ['/%asdf'],
        'conflicts': ['something:%unexpanded_conflicts'],
        'enhances': ['/%else'],
        'obsoletes': ['something:%unexpanded'],
        'recommends': ['/%unxpanded_recommends'],
        'supplements': ['/%something'],
        'arch': 'noarch',
        'name': 'unexpanded1',
        'version': '0',
        'release': '0',
    },
)


SelfPackage = get_tested_mock_package(
    lazyload=True,
    name='self',
    header={
        'requires': [
            'insserv',
            'rpmlib(CompressedFileNames) <= 3.0.4-1',
            'rpmlib(FileDigests) <= 4.6.0-1',
            'rpmlib(PayloadFilesHavePrefix) <= 4.0-1',
            'rpmlib(PayloadIsXz) <= 5.2-1',
            'xinetd',
        ],
        'provides': [
            'self',
            'self = 0-0',
            'self(x86-64) = 0-0',
        ],
        'arch': 'x86_64',
        'name': 'self',
        'version': '0',
        'release': '0',
    },
)


FuseCommonPackage = get_tested_mock_package(
    lazyload=True,
    name='fuse-common',
    files={
        'etc/fuse.conf': {
            'content': '# mount_max = 1000\nuser_allow_other\n',
            'flags': rpm.RPMFILE_NOREPLACE | rpm.RPMFILE_CONFIG,
        },
    },
    header={
        'requires': [
            'config(fuse-common) = 3.10.2-5.el8',
            'rpmlib(CompressedFileNames) <= 3.0.4-1',
            'rpmlib(FileDigests) <= 4.6.0-1',
            'rpmlib(PayloadFilesHavePrefix) <= 4.0-1',
            'rpmlib(PayloadIsXz) <= 5.2-1',
        ],
        'provides': [
            'config(fuse-common) = 3.10.2-5.el8',
            'fuse-common = 3.10.2-5.el8',
            'fuse-common = 3.2.1',
            'fuse-common = 3.3.0',
            'fuse-common(x86-64) = 3.10.2-5.el8',
        ],
        'arch': 'x86_64',
        'name': 'fuse-common',
        'version': '3.10.2',
        'release': '5.el8',
    },
)


FooDevelPackage = get_tested_mock_package(
    lazyload=True,
    name='foo-devel',
    header={
        'requires': [],
        'provides': [],
        'arch': 'x86_64',
        'name': 'foo-devel',
        'version': '0',
        'release': '0',
        'group': 'Games',
        'license': 'GPL-2.0+',
        'url': 'http://www.opensuse.org/',
        'buildhost': 'marxinbox.suse.cz',
        'summary': 'Lorem ipsum',
        'description': """Lorem ipsum dolor sit amet, consectetur adipisici elit, sed
eiusmod tempor incidunt ut labore et dolore magna aliqua. Ut enim
ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut
aliquid ex ea commodi consequat. Quis aute iure reprehenderit in
voluptate velit esse cillum dolore eu fugiat nulla pariatur.
Excepteur sint obcaecat cupiditat non proident, sunt in culpa qui
officia deserunt mollit anim id est laborum.
""",
        'changelogname': ['daniel.garcia@suse.com'],
        'changelogtime': [1303128000],
        'changelogtext': ['dummy'],
    },
)


MissingProvidesPackage = get_tested_mock_package(
    lazyload=True,
    name='missingprovides-devel',
    files=['/usr/lib64/pkgconfig/libparted.pc'],
    header={
        'requires': [],
        'provides': [],
        'arch': 'x86_64',
        'name': 'missingprovides-devel',
        'version': '0',
        'release': '0',
    },
)


InvalidExceptionPackage = MissingProvidesPackage.clone(
    extend=True,
    name='invalid-exception',
    header={
        'name': 'invalid-exception',
        'license': 'GPL-2.0+ WITH sparta',
    },
)


ValidExceptionPackage = InvalidExceptionPackage.clone(
    extend=True,
    name='valid-exception',
    header={
        'name': 'valid-exception',
        'license': 'GPL-2.0+ WITH 389-exception',
    },
)



from rpmlint.rpmdiff import Rpmdiff

from Testing import get_tested_path


def test_distribution_tags():
    oldpkg = get_tested_path('binary/mc-4.8.15-10.3.1.x86_64.rpm')
    newpkg = get_tested_path('binary/mc-4.8.21-2.1.x86_64.rpm')
    ignore = []
    diff = Rpmdiff(oldpkg, newpkg, ignore)
    textdiff = diff.textdiff()
    # the count always reports one less
    assert 231 <= len(textdiff.splitlines()) <= 233

    ignore.append('T')
    ignore.append('5')
    ignore.append('S')
    diff = Rpmdiff(oldpkg, newpkg, ignore)
    textdiff = diff.textdiff()
    assert 36 <= len(textdiff.splitlines()) <= 38

    assert 'added       /usr/share/mc/syntax/yaml.syntax' in textdiff


def test_exclude():
    oldpkg = get_tested_path('binary/mc-4.8.15-10.3.1.x86_64.rpm')
    newpkg = get_tested_path('binary/mc-4.8.21-2.1.x86_64.rpm')
    ignore = list('T5S')

    # print(Rpmdiff(oldpkg, newpkg, ignore=ignore).textdiff())

    for exclude in [], ['/usr/share/mc/ski'], ['/share/mc/skins'], ['skins']:
        diff = Rpmdiff(oldpkg, newpkg, ignore, exclude)
        textdiff = diff.textdiff()
        assert '/usr/share/mc/skins/yadt256.ini' in textdiff

    for exclude in (['/usr/share/mc/skins'], ['/usr/share/*/skins'],
                    ['/*/*/*/skins']):
        diff = Rpmdiff(oldpkg, newpkg, ignore, exclude)
        textdiff = diff.textdiff()
        assert '/usr/share/mc/skins/yadt256.ini' not in textdiff
        assert '/usr/share/mc/syntax/cuda.syntax' in textdiff

    for exclude in ['*.syntax'], ['syntax/cuda.syntax']:
        diff = Rpmdiff(oldpkg, newpkg, ignore, exclude)
        textdiff = diff.textdiff()
        assert '/usr/share/mc/skins/yadt256.ini' in textdiff
        assert '/usr/share/mc/syntax/cuda.syntax' not in textdiff



from mockdata.mock_zypp_syntax import (
    SyntaxAndPackage,
    SyntaxPackageandPackage,
)
import pytest
from rpmlint.checks.ZyppSyntaxCheck import ZyppSyntaxCheck
from rpmlint.filter import Filter

from Testing import CONFIG


@pytest.fixture(scope='function', autouse=True)
def zyppsyntaxcheck():
    CONFIG.info = True
    output = Filter(CONFIG)
    test = ZyppSyntaxCheck(CONFIG, output)
    return output, test


@pytest.fixture
def output(zyppsyntaxcheck):
    output, _test = zyppsyntaxcheck
    yield output


@pytest.fixture
def test(zyppsyntaxcheck):
    _output, test = zyppsyntaxcheck
    yield test


@pytest.mark.parametrize('package', [SyntaxPackageandPackage])
def test_packageand(package, test, output):
    test.check(package)
    out = output.print_results(output.results)
    assert 'suse-zypp-packageand packageand(c:d)' in out
    assert 'suse-zypp-packageand packageand(a:b)' in out
    assert '(a and b)' not in out
    assert 'bee' not in out
    assert 'suse-zypp-otherproviders otherproviders(yast2_theme)' in out


@pytest.mark.parametrize('package', [SyntaxAndPackage])
def test_packageand_ok(package, test, output):
    test.check(package)
    out = output.print_results(output.results)
    assert 'suse-zypp-packageand' not in out
    assert 'suse-zypp-otherproviders' not in out



import re

from pybeam import BeamFile
from rpm import expandMacro
from rpmlint.checks.AbstractCheck import AbstractFilesCheck
from rpmlint.helpers import byte_to_string


class ErlangCheck(AbstractFilesCheck):
    def __init__(self, config, output):
        super().__init__(config, output, r'.*?\.beam$')
        build_dir = expandMacro('%_builddir')
        self.source_re = re.compile(build_dir)

    def check_file(self, pkg, filename):
        try:
            beam = BeamFile(pkg.files[filename].path)

            if beam.compileinfo is None:
                self.output.add_info('W', pkg, 'beam-compile-info-missed', filename)
                return

            compile_state = byte_to_string(beam.compileinfo['source'])
            if 'debug_info' not in beam.compileinfo['options']:
                self.output.add_info('E', pkg, 'beam-compiled-without-debuginfo', filename)

            # This can't be an error as builddir can be user specific and vary between users
            # it could be error in OBS where all the builds are done by user abuild, not in
            # general.
            if not self.source_re.match(compile_state):
                self.output.add_info('W', pkg, 'beam-was-not-recompiled', filename, compile_state)
        except Exception:
            self.output.add_info('E', pkg, 'pybeam-failed', filename)



from mockdata.mock_sysvinitonsystemd import InitPackage, RcLinksPackage
import pytest
from rpmlint.checks.SysVInitOnSystemdCheck import SysVInitOnSystemdCheck
from rpmlint.filter import Filter

from Testing import CONFIG


@pytest.fixture(scope='function', autouse=True)
def sysvcheck():
    CONFIG.info = True
    output = Filter(CONFIG)
    test = SysVInitOnSystemdCheck(CONFIG, output)
    return output, test


@pytest.mark.parametrize('package', [InitPackage])
def test_sysv_init_on_systemd_check(package, sysvcheck):
    output, test = sysvcheck
    test.check(package)
    out = output.print_results(output.results)
    assert 'E: obsolete-insserv-requirement' in out
    assert 'E: deprecated-init-script weekly.script' in out
    assert 'E: deprecated-boot-script boot.script' in out


@pytest.mark.parametrize('package', [RcLinksPackage])
def test_overshadowing_of_initscript(package, sysvcheck):
    output, test = sysvcheck
    test.check(package)
    out = output.print_results(output.results)
    assert 'E: systemd-shadowed-initscript bar' in out



from Testing import get_tested_mock_package


SCRIPT1 = """#!/bin/sh
xz $tmp/1 &> /dev/null
"""

SCRIPT2 = """#!/bin/sh
do
"""


BashismsPackage = get_tested_mock_package(
    files={
        '/bin/script1': {'content': SCRIPT1},
        '/bin/script2': {'content': SCRIPT2},
    },
)



from Testing import get_tested_mock_package


I18NPackage = get_tested_mock_package(
    lazyload=True,
    files=[
        '/usr/share/locale/zh/LC_MESSAGES/goodvibes.mo',
        '/usr/share/locale/zh_Hant/LC_MESSAGES/goodvibes.mo',
        '/usr/share/locale/es_ES/LC_MESSAGES/goodvibes.mo',
        '/usr/share/locale/zh_TW/LC_MESSAGES/goodvibes.mo',
        '/usr/share/locale/pt_BR/LC_MESSAGES/goodvibes.mo',
    ],
)


InvalidI18NPackage = get_tested_mock_package(
    lazyload=True,
    files=[
        '/usr/share/locale/xx_ES/LC_MESSAGES/goodvibes.mo',
        '/usr/share/locale/es_XX/LC_MESSAGES/goodvibes.mo',
        '/usr/share/locale/xx/LC_MESSAGES/goodvibes.mo',
    ],
)



from mockdata.mock_files import Shlib2DevelPackage
import pytest
from rpmlint.checks.LibraryDependencyCheck import LibraryDependencyCheck
from rpmlint.filter import Filter

from Testing import CONFIG


@pytest.fixture(scope='function', autouse=True)
def libdependencycheck():
    CONFIG.info = True
    output = Filter(CONFIG)
    test = LibraryDependencyCheck(CONFIG, output)
    return output, test


@pytest.mark.parametrize('package', [Shlib2DevelPackage])
def test_shlib2_devel(package, libdependencycheck):
    output, test = libdependencycheck
    test.check(package)
    test.after_checks()
    out = output.print_results(output.results)
    print(out)
    assert 'E: no-library-dependency-for /usr/lib/libfoo.so.1' in out



from mockdata.mock_FHS import NoFHSPackage
import pytest
from rpmlint.checks.FHSCheck import FHSCheck
from rpmlint.filter import Filter

from Testing import CONFIG


@pytest.fixture(scope='function', autouse=True)
def fhscheck():
    CONFIG.info = True
    output = Filter(CONFIG)
    test = FHSCheck(CONFIG, output)
    return output, test


@pytest.mark.parametrize('package', [NoFHSPackage])
def test_FHS_compliance(package, fhscheck):
    """
    Check that the directories are not FHS compliant.
    """
    output, test = fhscheck
    test.check(package)
    out = output.print_results(output.results)

    # Check invalid /usr subdirectory
    assert 'non-standard-dir-in-usr' in out
    assert 'sbin' in out

    # Check invalid /var subdirectory
    assert 'non-standard-dir-in-var' in out
    assert 'lib' in out



from Testing import get_tested_mock_package


WrongIconSizePackage = get_tested_mock_package(
    files={
        '/usr/share/tasque/icons/hicolor/16x16/status/tasque-note.png': {
            'metadata': {'magic': '22 x 22'}
        }
    }
)



#!/usr/bin/python3

# Generate ISO codes for use with e.g. locale subdir checks
# http://alioth.debian.org/projects/pkg-isocodes/

import codecs
import json
import os
from pprint import pprint
import sys
from urllib.request import urlopen


iso_3166_1_url = os.environ.get('ISO_3166_1_URL', 'https://salsa.debian.org/iso-codes-team/iso-codes/raw/main/data/iso_3166-1.json')
iso_639_3_url = os.environ.get('ISO_639_3_URL', 'https://salsa.debian.org/iso-codes-team/iso-codes/raw/main/data/iso_639-3.json')
iso_639_2_url = os.environ.get('ISO_639_2_URL', 'https://salsa.debian.org/iso-codes-team/iso-codes/raw/main/data/iso_639-2.json')
iso_15924_url = os.environ.get('ISO_15924_URL', 'https://salsa.debian.org/iso-codes-team/iso-codes/raw/main/data/iso_15924.json')

langs = set()
countries = set()

# country codes (2 letters)
with urlopen(iso_3166_1_url) as f:
    data = json.load(codecs.getreader('utf-8')(f))
    for entry in data['3166-1']:
        countries.add(entry['alpha_2'])

# language codes (2 or 3 letters, 3 only for ones we don't have 2-letter one)
with urlopen(iso_639_3_url) as f:
    data = json.load(codecs.getreader('utf-8')(f))
    for entry in data['639-3']:
        langs.add(entry.get('alpha_2') or entry['alpha_3'])
# Need to check iso-639-2 for collective language codes not in iso-639-3
with urlopen(iso_639_2_url) as f:
    data = json.load(codecs.getreader('utf-8')(f))
    for entry in data['639-2']:
        entry_code = entry.get('alpha_2') or entry['alpha_3']
        if entry_code not in langs:
            langs.add(entry_code)
# ISO 15924, Codes for the representation of names of scripts
with urlopen(iso_15924_url) as f:
    data = json.load(codecs.getreader('utf-8')(f))
    for entry in data['15924']:
        countries.add(entry['alpha_4'])

print(f'# Generated with {sys.argv[0]}')
print('')
print('LANGUAGES = \\')
pprint(langs)
print('')
print('COUNTRIES = \\')
pprint(countries)



from mockdata.mock_shlib_policy import (
    Libslp1234Package,
    LibslpMissingSuffixPackage,
    Libtest1Package,
)
import pytest
from rpmlint.checks.SharedLibraryPolicyCheck import SharedLibraryPolicyCheck
from rpmlint.filter import Filter

from Testing import CONFIG


@pytest.fixture(scope='function', autouse=True)
def slpcheck():
    CONFIG.info = True
    output = Filter(CONFIG)
    test = SharedLibraryPolicyCheck(CONFIG, output)
    return output, test


@pytest.mark.parametrize('package', [Libtest1Package])
def test_shlib_policy_wrong_name(package, slpcheck):
    output, test = slpcheck
    test.check(package)
    out = output.print_results(output.results)
    assert 'W: shlib-unversioned-lib libtest.so.1x' in out


@pytest.mark.parametrize('package', [LibslpMissingSuffixPackage])
def test_shlib_policy_missing_suffix(package, slpcheck):
    output, test = slpcheck
    test.check(package)
    out = output.print_results(output.results)
    assert 'E: shlib-policy-excessive-dependency libsparta.so.2' in out


@pytest.mark.parametrize('package', [Libslp1234Package])
def test_shlib_policy_errors(package, slpcheck):
    output, test = slpcheck
    test.check(package)
    out = output.print_results(output.results)
    assert 'W: shlib-fixed-dependency libsparta.so.2 = 1.23' in out



from Testing import get_tested_mock_package


BAR_INITRD = """
#! /bin/bash

### BEGIN INIT INFO
# Provides:          foo
# Required-Start:    $local_fs $network
# Required-Stop:     $local_fs
# Default-Start:     2 3 4 5
# Default-Stop:      0 1 6
# Short-Description: foo service
# Description:       Run Foo service
### END INIT INFO

# Carry out specific functions when asked to by the system
case "$1" in
  start)
    echo "Starting Foo..."
    sudo -u foo-user bash -c 'cd /path/to/scripts/ && ./start-foo.sh'
    ;;
  stop)
    echo "Stopping Foo..."
    sudo -u foo-user bash -c 'cd /path/to/scripts/ && ./stop-foo.sh'
    sleep 2
    ;;
  *)
    echo "Usage: /etc/init.d/foo {start|stop}"
    exit 1
    ;;
esac

exit 0
"""


InitPackage = get_tested_mock_package(
    lazyload=True,
    files={
        '/etc/init.d/boot.script': {'content': ''},
        '/etc/init.d/weekly.script': {'content': ''},
    },
    header={'requires': ['insserv']},
)


RcLinksPackage = get_tested_mock_package(
    lazyload=True,
    files={
        '/etc/init.d/bar': {'content': BAR_INITRD},
        '/usr/lib/systemd/system/bar.service': {},
        '/usr/lib/systemd/system/foo.service': {},
    },
    header={'requires': ['insserv']},
)



import stat

import rpm

from Testing import get_tested_mock_package


MyDocPackage = get_tested_mock_package(
    lazyload=True,
    files={
        '/usr/share/doc/packages/mydoc/README': {
            'metadata': {
                'mode': 0o755 | stat.S_IFREG,
                'flags': rpm.RPMFILE_DOC,
            },
        },
        '/usr/share/doc/packages/mydoc/doc.html': {
            'metadata': {
                'mode': 0o755 | stat.S_IFREG,
                'flags': rpm.RPMFILE_DOC,
            },
        },
        '/usr/share/doc/packages/mydoc/strace.txt': {
            'metadata': {
                'mode': 0o644 | stat.S_IFREG,
                'flags': rpm.RPMFILE_DOC,
                'size': 268 * 1024,
            },
        },
    },
    header={
        'requires': [
            'python-leftover',
            'python-no-leftover',
        ],
    },
)


DocFileDependencyPackage = get_tested_mock_package(
    lazyload=True,
    name='doc-file-dependency',
    files={
        '/usr/bin/example': {
            'metadata': {'mode': stat.S_IFREG | 0o755, 'flags': rpm.RPMFILE_DOC},
        },
        '/usr/lib/python3.7/site-packages/__pycache__/example.cpython-37.pyc': {
            'metadata': {
                'mode': stat.S_IFREG | 0o644,
                'flags': rpm.RPMFILE_DOC,
                'requires': ['python(abi) = 3.7'],
            },
        },
        '/usr/lib/python3.7/site-packages/example-1.0.0-py3.7.egg-info/PKG-INFO': {
            'metadata': {'mode': stat.S_IFREG | 0o644, 'flags': rpm.RPMFILE_DOC},
        },
        '/usr/lib/python3.7/site-packages/example-1.0.0-py3.7.egg-info/SOURCES.txt': {
            'metadata': {'mode': stat.S_IFREG | 0o644, 'flags': rpm.RPMFILE_DOC},
        },
        '/usr/lib/python3.7/site-packages/example-1.0.0-py3.7.egg-info/dependency_links.txt': {
            'metadata': {'mode': stat.S_IFREG | 0o644, 'flags': rpm.RPMFILE_DOC},
        },
        '/usr/lib/python3.7/site-packages/example-1.0.0-py3.7.egg-info/entry_points.txt': {
            'metadata': {'mode': stat.S_IFREG | 0o644, 'flags': rpm.RPMFILE_DOC},
        },
        '/usr/lib/python3.7/site-packages/example-1.0.0-py3.7.egg-info/top_level.txt': {
            'metadata': {'mode': stat.S_IFREG | 0o644, 'flags': rpm.RPMFILE_DOC},
        },
        '/usr/lib/python3.7/site-packages/example.py': {
            'metadata': {
                'mode': stat.S_IFREG | 0o644,
                'flags': rpm.RPMFILE_DOC,
                'requires': ['python(abi) = 3.7'],
            },
        },
    },
    header={'provides': [], 'requires': []},
)

InstallFileInDocPackage = get_tested_mock_package(
    lazyload=True,
    name='install-file-in-docs',
    files={
        '/usr/share/doc/packages/install-file-in-docs/INSTALL': {
            'metadata': {'mode': stat.S_IFREG | 0o644, 'flags': rpm.RPMFILE_DOC},
        },
    },
    header={'provides': [], 'requires': []},
)



from mockdata.mock_python import (
    IPythonMissingRequirePackage,
    PythonDocFolderPackage,
    PythonDocModulePackage,
    PythonEggInfoFileackage,
    PythonFlitFedoraMockPackage,
    PythonFlitLeftoverRequirePackage,
    PythonFlitMissingRequirePackage,
    PythonFlitMockPackage,
    PythonIcecreamLeftoverRequirePackage,
    PythonIcecreamMissingRequirePackage,
    PythonIcecreamPackage,
    PythonJupyterEventsMockPackage,
    PythonJupyterServerFileidMockPackage,
    PythonMultiplePYCMockPackage,
    PythonScikitBuildMockPackage,
    PythonSinglePYCMockPackage,
)
import pytest
from rpmlint.checks.PythonCheck import PythonCheck
from rpmlint.filter import Filter

from Testing import CONFIG, get_tested_mock_package


@pytest.fixture(scope='function', autouse=True)
def pythoncheck():
    CONFIG.info = True
    output = Filter(CONFIG)
    test = PythonCheck(CONFIG, output)
    yield output, test


@pytest.fixture
def output(pythoncheck):
    output, _test = pythoncheck
    yield output


@pytest.fixture
def test(pythoncheck):
    _output, test = pythoncheck
    yield test


@pytest.mark.parametrize('package', [PythonDocFolderPackage])
def test_python_doc_in_package(package, test, output):
    test.check(package)
    out = output.print_results(output.results)
    assert 'W: python-doc-in-package /usr/lib/python2.7/site-packages/python-mypackage/doc' in out
    assert 'W: python-doc-in-package /usr/lib/python2.7/site-packages/python-mypackage/docs' in out
    assert 'W: python-doc-in-package /usr/lib/python3.10/site-packages/python-mypackage/doc' in out
    assert 'W: python-doc-in-package /usr/lib/python3.10/site-packages/python-mypackage/docs' in out
    assert 'W: python-doc-in-package /usr/lib64/python2.7/site-packages/python-mypackage/doc' in out
    assert 'W: python-doc-in-package /usr/lib64/python2.7/site-packages/python-mypackage/docs' in out
    assert 'W: python-doc-in-package /usr/lib64/python3.10/site-packages/python-mypackage/doc' in out
    assert 'W: python-doc-in-package /usr/lib64/python3.10/site-packages/python-mypackage/docs' in out


@pytest.mark.parametrize('package', [PythonDocModulePackage])
def test_python_doc_module_in_package(package, test, output):
    test.check(package)
    out = output.print_results(output.results)
    assert 'W: python-doc-in-package /usr/lib/python2.7/site-packages/python-mypackage/doc' not in out
    assert 'W: python-doc-in-package /usr/lib/python2.7/site-packages/python-mypackage/docs' not in out
    assert 'W: python-doc-in-package /usr/lib/python3.10/site-packages/python-mypackage/doc' not in out
    assert 'W: python-doc-in-package /usr/lib/python3.10/site-packages/python-mypackage/docs' not in out
    assert 'W: python-doc-in-package /usr/lib64/python2.7/site-packages/python-mypackage/doc' not in out
    assert 'W: python-doc-in-package /usr/lib64/python2.7/site-packages/python-mypackage/docs' not in out
    assert 'W: python-doc-in-package /usr/lib64/python3.10/site-packages/python-mypackage/doc' not in out
    assert 'W: python-doc-in-package /usr/lib64/python3.10/site-packages/python-mypackage/docs' not in out


@pytest.mark.parametrize('package', [PythonEggInfoFileackage])
def test_python_distutils_egg_info(package, test, output):
    test.check(package)
    out = output.print_results(output.results)
    assert 'E: python-egg-info-distutils-style /usr/lib/python2.7/site-packages/mydistutilspackage.egg-info' in out
    assert 'E: python-egg-info-distutils-style /usr/lib/python3.10/site-packages/mydistutilspackage.egg-info' in out
    assert 'E: python-egg-info-distutils-style /usr/lib64/python2.7/site-packages/mydistutilspackage.egg-info' in out
    assert 'E: python-egg-info-distutils-style /usr/lib64/python3.10/site-packages/mydistutilspackage.egg-info' in out


@pytest.mark.parametrize('package', [get_tested_mock_package(
    files=[
        '/usr/lib/python2.7/site-packages/doc',
        '/usr/lib/python2.7/site-packages/docs',
        '/usr/lib/python3.10/site-packages/doc',
        '/usr/lib/python3.10/site-packages/docs',
        '/usr/lib64/python2.7/site-packages/doc',
        '/usr/lib64/python2.7/site-packages/docs',
        '/usr/lib64/python3.10/site-packages/doc',
        '/usr/lib64/python3.10/site-packages/docs',
    ]
)])
def test_python_doc_in_site_packages(package, test, output):
    test.check(package)
    out = output.print_results(output.results)
    assert 'E: python-doc-in-site-packages /usr/lib/python2.7/site-packages/doc' in out
    assert 'E: python-doc-in-site-packages /usr/lib/python2.7/site-packages/docs' in out
    assert 'E: python-doc-in-site-packages /usr/lib/python3.10/site-packages/doc' in out
    assert 'E: python-doc-in-site-packages /usr/lib/python3.10/site-packages/docs' in out
    assert 'E: python-doc-in-site-packages /usr/lib64/python2.7/site-packages/doc' in out
    assert 'E: python-doc-in-site-packages /usr/lib64/python2.7/site-packages/docs' in out
    assert 'E: python-doc-in-site-packages /usr/lib64/python3.10/site-packages/doc' in out
    assert 'E: python-doc-in-site-packages /usr/lib64/python3.10/site-packages/docs' in out


@pytest.mark.parametrize('package', [get_tested_mock_package(
    files=[
        '/usr/lib/python2.7/site-packages/src',
        '/usr/lib/python3.10/site-packages/src',
        '/usr/lib64/python2.7/site-packages/src',
        '/usr/lib64/python3.10/site-packages/src',
    ]
)])
def test_python_src_in_site_packages(package, test, output):
    test.check(package)
    out = output.print_results(output.results)
    assert 'E: python-src-in-site-packages /usr/lib/python2.7/site-packages/src' in out
    assert 'E: python-src-in-site-packages /usr/lib/python3.10/site-packages/src' in out
    assert 'E: python-src-in-site-packages /usr/lib64/python2.7/site-packages/src' in out
    assert 'E: python-src-in-site-packages /usr/lib64/python3.10/site-packages/src' in out


@pytest.mark.parametrize('package', [get_tested_mock_package(
    files=[
        '/usr/lib/python2.7/site-packages/test',
        '/usr/lib/python2.7/site-packages/tests',
        '/usr/lib/python3.10/site-packages/test',
        '/usr/lib/python3.10/site-packages/tests',
        '/usr/lib64/python2.7/site-packages/test',
        '/usr/lib64/python2.7/site-packages/tests',
        '/usr/lib64/python3.10/site-packages/test',
        '/usr/lib64/python3.10/site-packages/tests',
    ]
)])
def test_python_tests_in_site_packages(package, test, output):
    test.check(package)
    out = output.print_results(output.results)
    assert 'E: python-tests-in-site-packages /usr/lib/python2.7/site-packages/test' in out
    assert 'E: python-tests-in-site-packages /usr/lib/python2.7/site-packages/tests' in out
    assert 'E: python-tests-in-site-packages /usr/lib/python3.10/site-packages/test' in out
    assert 'E: python-tests-in-site-packages /usr/lib/python3.10/site-packages/tests' in out
    assert 'E: python-tests-in-site-packages /usr/lib64/python2.7/site-packages/test' in out
    assert 'E: python-tests-in-site-packages /usr/lib64/python2.7/site-packages/tests' in out
    assert 'E: python-tests-in-site-packages /usr/lib64/python3.10/site-packages/test' in out
    assert 'E: python-tests-in-site-packages /usr/lib64/python3.10/site-packages/tests' in out


@pytest.mark.parametrize('package', [PythonFlitMockPackage,
                                     PythonJupyterServerFileidMockPackage,
                                     PythonJupyterEventsMockPackage,
                                     PythonScikitBuildMockPackage,
                                     PythonFlitFedoraMockPackage,])
def test_python_dependencies_metadata(package, test, output):
    test.check(package)
    out = output.print_results(output.results)
    assert 'W: python-missing-require' not in out
    assert 'W: python-leftover-require' not in out


@pytest.mark.parametrize('package', [PythonIcecreamPackage])
def test_python_dependencies_requires(package, test, output):
    test.check(package)
    out = output.print_results(output.results)
    assert 'W: python-missing-require' not in out
    assert 'W: python-leftover-require' not in out


@pytest.mark.parametrize('package', [PythonIcecreamMissingRequirePackage])
def test_python_dependencies_missing_requires(package, test, output):
    test.check(package)
    out = output.print_results(output.results)
    assert 'W: python-missing-require' in out


@pytest.mark.parametrize('package', [PythonFlitMissingRequirePackage])
def test_python_dependencies_missing_metadata(package, test, output):
    test.check(package)
    out = output.print_results(output.results)
    assert 'W: python-missing-require' in out


@pytest.mark.parametrize('package', [
    PythonIcecreamLeftoverRequirePackage,
    PythonFlitLeftoverRequirePackage,
])
def test_python_dependencies_leftover(package, test, output):
    test.check(package)
    out = output.print_results(output.results)
    assert 'W: python-leftover-require' in out


@pytest.mark.parametrize('package', [PythonMultiplePYCMockPackage])
def test_python_pyc_multiple_versions(package, test, output):
    test.check(package)
    out = output.print_results(output.results)
    assert 'W: python-pyc-multiple-versions expected: 310' in out


@pytest.mark.parametrize('package', [PythonSinglePYCMockPackage])
def test_python_pyc_single_version(package, test, output):
    test.check(package)
    out = output.print_results(output.results)
    assert 'W: python-pyc-multiple-versions' not in out


@pytest.mark.parametrize('package', [
    get_tested_mock_package(files=['/usr/share/doc/packages/python-blinker-doc/.doctrees']),
    get_tested_mock_package(files=['/usr/lib/python3.11/site-packages/python-blinker/.doctrees']),
])
def test_python_sphinx_doctrees_leftover_warn(package, output, test):
    test.check(package)
    out = output.print_results(output.results)
    assert 'W: python-sphinx-doctrees-leftover' in out


@pytest.mark.parametrize('package', [
    get_tested_mock_package(files=['/usr/lib/python3.11/site-packages/python-blinker/doctrees.py']),
    get_tested_mock_package(files=['/usr/share/doc/packages/python-blinker-doc/doctrees']),
    get_tested_mock_package(files=['/usr/share/doc/packages/python-blinker-doc/.doctrees.html']),
])
def test_python_sphinx_doctrees_leftover_nowarn(package, output, test):
    test.check(package)
    out = output.print_results(output.results)
    assert 'W: python-sphinx-doctrees-leftover' not in out


@pytest.mark.parametrize('package', [IPythonMissingRequirePackage])
def test_python_dependencies_ipython(package, test, output):
    test.check(package)
    out = output.print_results(output.results)

    requirements = [
        'backcall',
        'decorator',
        'jedi',
        'matplotlib-inline',
        'pickleshare',
        'prompt_toolkit',
        'pygments',
        'stack_data',
        'traitlets',
    ]

    for req in requirements:
        assert f'W: python-missing-require {req}' in out

    # typing_extensions is in section [:python_version < "3.10"]
    assert 'W: python-missing-require typing_extensions' not in out

    # req312 is in section [:python_version > "3.11"]
    assert 'W: python-missing-require req312' in out

    # leftover is in section [:python_version < "3.10"]
    assert 'W: python-leftover-require python-leftover' in out
    assert 'W: python-leftover-require python-no-leftover' not in out



# coding=utf-8
"""
Represents common constants and classes for all platforms.

@group Power Source Type: POWER_TYPE_AC, POWER_TYPE_BATTERY, POWER_TYPE_UPS
@var POWER_TYPE_AC: The system is connected to the external power source.
@var POWER_TYPE_BATTERY: The system is connected to the battery.
@var POWER_TYPE_UPS: The system is connected to UPS.
@type POWER_TYPE_BATTERY: int
@type POWER_TYPE_AC: int
@type POWER_TYPE_UPS: int

@group Low Battery Warning Levels: LOW_BATTERY_WARNING_NONE, LOW_BATTERY_WARNING_EARLY, LOW_BATTERY_WARNING_FINAL
@var LOW_BATTERY_WARNING_NONE: The system is connected to the unlimited power source.
@var LOW_BATTERY_WARNING_EARLY: The battery has dropped below 22% remaining power.
@var LOW_BATTERY_WARNING_FINAL: The battery can provide no more than 10 minutes of runtime.
@type LOW_BATTERY_WARNING_EARLY: int
@type LOW_BATTERY_WARNING_NONE: int
@type LOW_BATTERY_WARNING_FINAL: int

@group Special Values For Time Remaining: TIME_REMAINING_UNKNOWN, TIME_REMAINING_UNLIMITED
@var TIME_REMAINING_UNKNOWN: Indicates the system is connected to a limited power source, but system is still
    calculating a time remaining estimate.
@var TIME_REMAINING_UNLIMITED: Indicates that the system is connected to an external power source, without time limit.
@type TIME_REMAINING_UNKNOWN: float
@type TIME_REMAINING_UNLIMITED: float
"""
__author__ = 'kulakov.ilya@gmail.com'

from abc import ABCMeta, abstractmethod
import weakref

__all__ = [
    'POWER_TYPE_AC',
    'POWER_TYPE_BATTERY',
    'POWER_TYPE_UPS',
    'LOW_BATTERY_WARNING_NONE',
    'LOW_BATTERY_WARNING_EARLY',
    'LOW_BATTERY_WARNING_FINAL',
    'TIME_REMAINING_UNKNOWN',
    'TIME_REMAINING_UNLIMITED',
    'PowerManagementObserver'
    ]


POWER_TYPE_AC = 0

POWER_TYPE_BATTERY = 1

POWER_TYPE_UPS = 2


LOW_BATTERY_WARNING_NONE = 1

LOW_BATTERY_WARNING_EARLY = 2

LOW_BATTERY_WARNING_FINAL = 3


TIME_REMAINING_UNKNOWN = -1.0

TIME_REMAINING_UNLIMITED = -2.0


class PowerManagementBase(object):
    """
    Base class for platform dependent PowerManagement functions.

    @ivar _weak_observers: List of weak reference to added observers
    @note: Platform's implementation may provide additional parameters for initialization
    """
    __metaclass__ = ABCMeta

    def __init__(self):
        self._weak_observers = []

    @abstractmethod
    def get_providing_power_source_type(self):
        """
        Returns type of the providing power source.

        @return: Possible values:
                - POWER_TYPE_AC
                - POWER_TYPE_BATTERY
                - POWER_TYPE_UPS
        @rtype: int
        """
        pass

    @abstractmethod
    def get_low_battery_warning_level(self):
        """
        Returns the system battery warning level.

        @return: Possible values:
            - LOW_BATTERY_WARNING_NONE
            - LOW_BATTERY_WARNING_EARLY
            - LOW_BATTERY_WARNING_FINAL
        @rtype: int
        """
        pass

    @abstractmethod
    def get_time_remaining_estimate(self):
        """
        Returns the estimated minutes remaining until all power sources (battery and/or UPS) are empty.

        @return: Special values:
            - TIME_REMAINING_UNKNOWN
            - TIME_REMAINING_UNLIMITED
        @rtype: float
        """
        pass

    @abstractmethod
    def add_observer(self, observer):
        """
        Adds weak ref to an observer.

        @param observer: Instance of class registered with PowerManagementObserver
        @raise TypeError: If observer is not registered with PowerManagementObserver abstract class
        """
        if not isinstance(observer, PowerManagementObserver):
            raise TypeError("observer MUST conform to power.PowerManagementObserver")
        self._weak_observers.append(weakref.ref(observer))

    @abstractmethod
    def remove_observer(self, observer):
        """
        Removes an observer.

        @param observer: Previously added observer
        """
        self._weak_observers.remove(weakref.ref(observer))

    def remove_all_observers(self):
        """
        Removes all registered observers.
        """
        for weak_observer in self._weak_observers:
            observer = weak_observer()
            if observer:
                self.remove_observer(observer)


class PowerManagementObserver:
    """
    Base class for PowerManagement observers.
    Do not make assumptions in what thread or event loop these methods are called.
    """
    __metaclass__ = ABCMeta

    @abstractmethod
    def on_power_sources_change(self, power_management):
        """
        @param power_management: Instance of PowerManagement posted notification
        """
        pass

    @abstractmethod
    def on_time_remaining_change(self, power_management):
        """
        @param power_management: Instance of PowerManagement posted notification
        """
        pass


class PowerManagementNoop(PowerManagementBase):
    """
    No-op subclass of PowerManagement.
    It operates like AC is always attached and power sources are never changed.
    """
    def get_providing_power_source_type(self):
        """
        @return: Always POWER_TYPE_AC
        """
        return POWER_TYPE_AC

    def get_low_battery_warning_level(self):
        """
        @return: Always LOW_BATTERY_WARNING_NONE
        """
        return LOW_BATTERY_WARNING_NONE

    def get_time_remaining_estimate(self):
        """
        @return: Always TIME_REMAINING_UNLIMITED
        """
        return TIME_REMAINING_UNLIMITED

    def add_observer(self, observer):
        """
        Does nothing.
        """
        pass

    def remove_observer(self, observer):
        """
        Does nothing.
        """
        pass

    def remove_all_observers(self):
        """
        Does nothing.
        """
        pass



#############################################################################
# Project         : Mandriva Linux
# Module          : rpmlint
# File            : InitScriptCheck.py
# Author          : Frederic Lepied
# Created On      : Fri Aug 25 09:26:37 2000
# Purpose         : check init scripts (files in /etc/rc.d/init.d)
#############################################################################

from pathlib import Path
import re

import rpm
from rpmlint.checks.AbstractCheck import AbstractCheck
from rpmlint.helpers import readlines


chkconfig_content_regex = re.compile(r'^\s*#\s*chkconfig:\s*([-0-9]+)\s+[-0-9]+\s+[-0-9]+')
subsys_regex = re.compile(r'/var/lock/subsys/([^/"\'\s;&|]+)', re.MULTILINE)
chkconfig_regex = re.compile(r'^[^#]*(chkconfig|add-service|del-service)', re.MULTILINE)
status_regex = re.compile(r'^[^#]*status', re.MULTILINE)
reload_regex = re.compile(r'^[^#]*reload', re.MULTILINE)
lsb_tags_regex = re.compile(r'^# ([\w-]+):\s*(.*?)\s*$')
lsb_cont_regex = re.compile(r'^#(?:%s|  )(.*?)\s*$' % '\t')

LSB_KEYWORDS = ('Provides', 'Required-Start', 'Required-Stop', 'Should-Start',
                'Should-Stop', 'Default-Start', 'Default-Stop',
                'Short-Description', 'Description')
RECOMMENDED_LSB_KEYWORDS = ('Provides', 'Required-Start', 'Required-Stop',
                            'Default-Stop', 'Short-Description')


var_regex = re.compile(r'^(.*)\${?(\w+)}?(.*)$')


def shell_var_value(var, script):
    assign_regex = re.compile(r'\b' + re.escape(var) +
                              r'\s*=\s*(.+)\s*(#.*)*$', re.MULTILINE)
    res = assign_regex.search(script)
    if res:
        res2 = var_regex.search(res.group(1))
        if res2 and res2.group(2) == var:  # infinite loop
            return None
        return substitute_shell_vars(res.group(1), script)
    else:
        return None


def substitute_shell_vars(val, script):
    res = var_regex.search(val)
    if res:
        value = shell_var_value(res.group(2), script)
        if not value:
            value = ''
        return res.group(1) + value + \
            substitute_shell_vars(res.group(3), script)
    else:
        return val


class InitScriptCheck(AbstractCheck):
    def __init__(self, config, output):
        super().__init__(config, output)
        self.use_deflevels = self.config.configuration['UseDefaultRunlevels']
        self.use_subsys = self.config.configuration['UseVarLockSubsys']

    def check_binary(self, pkg):
        initscript_list = []
        for fname, pkgfile in pkg.files.items():
            if not fname.startswith('/etc/init.d/') and \
                    not fname.startswith('/etc/rc.d/init.d/'):
                continue

            basename = Path(fname).name
            initscript_list.append(basename)
            if pkgfile.mode & 0o500 != 0o500:
                self.output.add_info('E', pkg, 'init-script-non-executable', fname)

            if '.' in basename:
                self.output.add_info('E', pkg, 'init-script-name-with-dot', fname)

            # check chkconfig call in %post and %preun
            postin = pkg[rpm.RPMTAG_POSTIN] or \
                pkg.scriptprog(rpm.RPMTAG_POSTINPROG)
            if not postin:
                self.output.add_info('E', pkg, 'init-script-without-chkconfig-postin', fname)
            elif not chkconfig_regex.search(postin):
                self.output.add_info('E', pkg, 'postin-without-chkconfig', fname)

            preun = pkg[rpm.RPMTAG_PREUN] or \
                pkg.scriptprog(rpm.RPMTAG_PREUNPROG)
            if not preun:
                self.output.add_info('E', pkg, 'init-script-without-chkconfig-preun', fname)
            elif not chkconfig_regex.search(preun):
                self.output.add_info('E', pkg, 'preun-without-chkconfig', fname)

            status_found = False
            reload_found = False
            chkconfig_content_found = False
            subsys_regex_found = False
            in_lsb_tag = False
            in_lsb_description = False
            lastline = ''
            lsb_tags = {}
            # check common error in file content
            content = None
            try:
                content = list(readlines(pkgfile.path))
            except Exception as e:
                self.output.add_info('W', pkg, 'read-error', e)
                continue
            content_str = ''.join(content)
            for line in content:
                line = line[:-1]  # chomp
                # TODO check if there is only one line like this
                if line.startswith('### BEGIN INIT INFO'):
                    in_lsb_tag = True
                    continue
                if line.endswith('### END INIT INFO'):
                    in_lsb_tag = False
                    for kw, vals in lsb_tags.items():
                        if len(vals) != 1:
                            self.output.add_info('E', pkg, 'redundant-lsb-keyword', kw)

                    for kw in RECOMMENDED_LSB_KEYWORDS:
                        if kw not in lsb_tags:
                            self.output.add_info('W', pkg, 'missing-lsb-keyword',
                                                 f'{kw} in {fname}')
                if in_lsb_tag:
                    # TODO maybe we do not have to handle this ?
                    if lastline.endswith('\\'):
                        line = lastline + line
                    else:
                        res = lsb_tags_regex.search(line)
                        if not res:
                            cres = lsb_cont_regex.search(line)
                            if not (in_lsb_description and cres):
                                in_lsb_description = False
                                self.output.add_info('E',
                                                     pkg, 'malformed-line-in-lsb-comment-block',
                                                     line)
                            else:
                                lsb_tags['Description'][-1] += \
                                    ' ' + cres.group(1)
                        else:
                            tag = res.group(1)
                            if not tag.startswith('X-') and \
                                    tag not in LSB_KEYWORDS:
                                self.output.add_info('E', pkg, 'unknown-lsb-keyword', line)
                            else:
                                in_lsb_description = (tag == 'Description')
                                if tag not in lsb_tags:
                                    lsb_tags[tag] = []
                                lsb_tags[tag].append(res.group(2))
                    lastline = line

                if not status_found and status_regex.search(line):
                    status_found = True

                if not reload_found and reload_regex.search(line):
                    reload_found = True

                res = chkconfig_content_regex.search(line)
                if res:
                    chkconfig_content_found = True
                    if self.use_deflevels:
                        if res.group(1) == '-':
                            self.output.add_info('W', pkg, 'no-default-runlevel', fname)
                    elif res.group(1) != '-':
                        self.output.add_info('W', pkg, 'service-default-enabled', fname)

                res = subsys_regex.search(line)
                if res:
                    subsys_regex_found = True
                    name = res.group(1)
                    if self.use_subsys and name != basename:
                        error = True
                        if name[0] == '$':
                            value = substitute_shell_vars(name, content_str)
                            if value == basename:
                                error = False
                        else:
                            i = name.find('}')
                            if i != -1:
                                name = name[0:i]
                                error = name != basename
                        if error and len(name):
                            if name[0] == '$':
                                self.output.add_info('W', pkg, 'incoherent-subsys', fname,
                                                     name)
                            else:
                                self.output.add_info('E', pkg, 'incoherent-subsys', fname,
                                                     name)

            if 'Default-Start' in lsb_tags and ''.join(lsb_tags['Default-Start']):
                self.output.add_info('W', pkg, 'service-default-enabled', fname)

            if not status_found:
                self.output.add_info('E', pkg, 'no-status-entry', fname)
            if not reload_found:
                self.output.add_info('W', pkg, 'no-reload-entry', fname)
            if not chkconfig_content_found:
                self.output.add_info('E', pkg, 'no-chkconfig-line', fname)
            if not subsys_regex_found and self.use_subsys:
                self.output.add_info('E', pkg, 'subsys-not-used', fname)
            elif subsys_regex_found and not self.use_subsys:
                self.output.add_info('E', pkg, 'subsys-unsupported', fname)

        if len(initscript_list) == 1:
            pkgname = re.sub('-sysvinit$', '', pkg.name.lower())
            goodnames = (pkgname, pkgname + 'd')
            if initscript_list[0] not in goodnames:
                self.output.add_info('W', pkg, 'incoherent-init-script-name',
                                     initscript_list[0], str(goodnames))



import os
import subprocess
import unittest.mock as mock

import pytest
import rpm
from rpmlint.pkg import parse_deps, rangeCompare

from Testing import get_tested_package


def test_parse_deps():
    for (arg, exp) in (
        ('a, b < 1.0 c = 5:2.0-3 d',
         [('a', 0, (None, None, None)),
          ('b', rpm.RPMSENSE_LESS, (None, '1.0', None)),
          ('c', rpm.RPMSENSE_EQUAL, (5, '2.0', '3')),
          ('d', 0, (None, None, None))]),
    ):
        assert parse_deps(arg) == exp


def test_range_compare():
    for (req, prov) in (
        (('foo', rpm.RPMSENSE_LESS, (None, '1.0', None)),
         ('foo', rpm.RPMSENSE_EQUAL, (1, '0.5', None))),
    ):
        assert not rangeCompare(req, prov)


@pytest.mark.parametrize('package', ['binary/python311-pytest-xprocess'])
@pytest.mark.skipif(os.getuid() == 0, reason='Root has full permission')
def test_extract_fail(package, tmp_path):
    """
    Check that rpm2cpio fails to extract this package because it has no
    permissions to some files.
    """

    with mock.patch('shutil.which') as mock_which:
        mock_which.return_value = None
        # the package cannot be extracted using rpm2cpio because it contains a directory without 'x' permission
        with pytest.raises(subprocess.CalledProcessError) as exc:
            get_tested_package(package, tmp_path)
        mock_which.assert_called_once_with('rpm2archive')
        # check that it was rpm2cpio what failed
        assert exc.match(r'rpm2cpio .*')



from pathlib import Path

import pytest
from rpmlint.config import Config

from Testing import get_tested_path, TEST_CONFIG


TEST_CONFIG_2 = [get_tested_path('configs/test2.config')]
TEST_CONFIG_FILTERS = [get_tested_path('configs/testfilters.config')]
TEST_LIST1 = [get_tested_path('configs/testlists1.config')]
TEST_LIST2 = [get_tested_path('configs/testlists2.config')]
TEST_OVERRIDE = [get_tested_path('configs/test.override.config')]
TEST_RPMLINTRC = get_tested_path('configs/testing-rpmlintrc')
TEST_BROKEN = [get_tested_path('configs/broken.config')]


def test_printing(capsys):
    cfg = Config()
    cfg.print_config()
    out, err = capsys.readouterr()
    assert not err
    assert out


def test_custom_config(capsys):
    cfg = Config()
    # bullshit config
    cfg.find_configs([Path('BULLSHIT')])
    out, err = capsys.readouterr()
    assert Path('BULLSHIT') not in cfg.conf_files
    assert 'BULLSHIT' in err
    # existing config
    cfg.find_configs(TEST_CONFIG)
    out, err = capsys.readouterr()
    assert cfg.conf_files
    assert not err


def test_broken_config(capsys):
    with pytest.raises(SystemExit) as pytest_wrapped_e:
        Config(TEST_BROKEN)
    assert pytest_wrapped_e.type is SystemExit
    assert pytest_wrapped_e.value.code == 4


def test_parsing():
    # ugly content variables from old config
    bad_crypto_warning = \
        """This application package calls a function to explicitly set crypto ciphers
        for SSL/TLS. That may cause the application not to use the system-wide set
        cryptographic policy and should be modified in accordance to:
        https://fedoraproject.org/wiki/Packaging:CryptoPolicies"""
    forbidden_functions = {
        'crypto-policy-non-compliance-openssl': {
            'f_name': 'SSL_CTX_set_cipher_list',
            'description': bad_crypto_warning,
        },
        'crypto-policy-non-compliance-gnutls-1': {
            'f_name': 'gnutls_priority_set_direct',
            'description': bad_crypto_warning,
        },
        'crypto-policy-non-compliance-gnutls-2': {
            'f_name': 'gnutls_priority_init',
            'good_param': 'SYSLOG',
            'description': bad_crypto_warning
        },
    }
    cfg = Config(TEST_CONFIG)
    assert cfg.configuration
    assert cfg.configuration['Distribution'] == 'Fedora Project'
    assert cfg.configuration['WarnOnFunction'] == forbidden_functions
    # default value check
    assert cfg.configuration['UseDefaultRunlevels'] is True


def test_double_config():
    """
    Load two configs and make sure we properly load all the values
    """
    cfg = Config(TEST_CONFIG)
    assert len(cfg.conf_files) == 2
    assert cfg.configuration['ExtraMenuNeeds'][0] == 'gnome'
    # shovel in another config
    cfg.load_config(TEST_CONFIG_2)
    assert len(cfg.conf_files) == 3
    assert cfg.configuration['ExtraMenuNeeds'][-1] == 'windows'
    assert cfg.configuration['WarnOnFunction']['crypto-policy-non-compliance-openssl']['f_name'] == 'REPLACED'
    assert cfg.configuration['WarnOnFunction']['crypto-policy-3']['f_name'] == 'new_blobie'


def test_filters():
    """
    Load some filters and make sure we generate nice regexp
    """
    cfg = Config(TEST_CONFIG_FILTERS)
    assert len(cfg.configuration['Filters']) == 12
    assert cfg.configuration['Filters'][0] == '.*invalid-buildhost.*'


def test_list_merging():
    """
    Load two configs and check we loaded up in proper older with
    replacing based on TOML syntax
    """
    cfg = Config(TEST_LIST1)
    assert len(cfg.configuration['Filters']) == 1
    assert cfg.configuration['ValidGroups'][0] == 'bullshitgroup'
    cfg.load_config(TEST_LIST2)
    assert len(cfg.conf_files) == 3
    assert len(cfg.configuration['Filters']) == 2
    assert len(cfg.configuration['ValidGroups']) == 3
    assert cfg.configuration['ValidGroups'][2] == 'System/Libraries'
    cfg.load_config(TEST_OVERRIDE)
    assert len(cfg.configuration['ValidGroups']) == 1


def test_badness_functions():
    """
    Test badness settings
    """
    cfg = Config(TEST_CONFIG_2)
    assert len(cfg.configuration['Scoring']) == 0
    cfg.set_badness('suse-dbus-unauthorized-service', 15)
    assert len(cfg.configuration['Scoring']) == 1
    assert cfg.configuration['Scoring']['suse-dbus-unauthorized-service'] == 15


def test_rpmlint_loading():
    """
    Make sure we can load up rpmlintrc file without executing any code
    """
    cfg = Config(TEST_CONFIG)
    cfg.load_rpmlintrc(TEST_RPMLINTRC)
    assert 'arch-independent-package-contains-binary-or-object ' in cfg.configuration['Filters']
    assert len(cfg.configuration['Filters']) == 113
    assert len(cfg.configuration['Scoring']) == 3



from rpmlint.checks.AbstractCheck import AbstractCheck


class XinetdDepCheck(AbstractCheck):
    def check(self, pkg):
        if pkg.is_source:
            return

        for req in pkg.requires + pkg.prereq:
            if req[0] == 'xinetd':
                self.output.add_info('E', pkg, 'obsolete-xinetd-requirement')



from rpm import RPMFILE_CONFIG, RPMFILE_NOREPLACE

from Testing import get_tested_mock_package


ConfigFilesBrokenPackage = get_tested_mock_package(
    lazyload=True,
    files={
        '/etc/conffile1': {'metadata': {'flags': RPMFILE_CONFIG}},
        '/var/conffile2': {'metadata': {'flags': RPMFILE_CONFIG}},
        '/usr/share/conffile3': {'metadata': {'flags': RPMFILE_CONFIG}},
    }
)


ConfigFilesOkPackage = get_tested_mock_package(
    lazyload=True,
    files={
        '/etc/conffile1': {'metadata': {'flags': RPMFILE_CONFIG & RPMFILE_NOREPLACE}},
        '/var/conffile2': {'metadata': {'flags': RPMFILE_CONFIG & RPMFILE_NOREPLACE}},
    }
)



from pathlib import Path

import pytest
from rpmlint.checks.BinariesCheck import BinariesCheck
from rpmlint.filter import Filter
from rpmlint.lddparser import LddParser
from rpmlint.pkg import FakePkg, get_magic

from Testing import CONFIG, get_tested_path, IS_X86_64


@pytest.fixture(scope='function', autouse=True)
def binariescheck():
    CONFIG.info = True
    output = Filter(CONFIG)
    test = BinariesCheck(CONFIG, output)
    return output, test


def get_full_path(path):
    return str(get_tested_path(Path('ldd', path)))


def lddparser(path, system_path=None):
    if system_path is None:
        system_path = path
    return LddParser(get_full_path(path), system_path, True)


def run_elf_checks(test, pkg, pkgfile):
    test._detect_attributes(get_magic(pkgfile.path))
    test.run_elf_checks(pkg, pkgfile)


@pytest.mark.skipif(not IS_X86_64, reason='x86-64 only')
def test_unused_dependency():
    ldd = lddparser('libtirpc.so.3.0.0')
    assert not ldd.parsing_failed_reason
    assert len(ldd.unused_dependencies) >= 1
    assert 'liXXXsapi_krb5.so.2' in ldd.unused_dependencies


@pytest.mark.skipif(not IS_X86_64, reason='x86-64 only')
def test_undefined_symbol():
    ldd = lddparser('libtirpc.so.3.0.0')
    assert not ldd.parsing_failed_reason
    assert len(ldd.undefined_symbols) >= 22
    assert 'GSS_C_NT_HOSTBASED_SERVICE' in ldd.undefined_symbols


def test_ldd_parser_failure():
    ldd = lddparser('not-existing-file')
    assert 'not-existing-file: No such file or directory' in ldd.parsing_failed_reason


@pytest.mark.skipif(not IS_X86_64, reason='x86-64 only')
def test_dependencies():
    ldd = lddparser('libtirpc.so.3.0.0')
    assert not ldd.parsing_failed_reason
    assert len(ldd.dependencies) == 5
    assert any(d for d in ldd.dependencies if d.startswith('linux-vdso.so.1'))


@pytest.mark.skipif(not IS_X86_64, reason='x86-64 only')
def test_unused_dependency_in_package(binariescheck):
    output, test = binariescheck
    with FakePkg('fake') as pkg:
        pkgfile = pkg.add_file(get_full_path('libtirpc.so.3.0.0'), '/lib64/x.so')
        run_elf_checks(test, pkg, pkgfile)
        assert not test.readelf_parser.parsing_failed_reason()
        assert not test.ldd_parser.parsing_failed_reason
        out = output.print_results(output.results)
        assert 'E: unused-direct-shlib-dependency ' in out


@pytest.mark.skipif(not IS_X86_64, reason='x86-64 only')
def test_unused_dependency_in_package_for_executable(binariescheck):
    output, test = binariescheck
    with FakePkg('fake') as pkg:
        pkgfile = pkg.add_file(get_full_path('appletviewer'), '/usr/bin/appletviewer')
        run_elf_checks(test, pkg, pkgfile)
        assert not test.readelf_parser.parsing_failed_reason()
        assert not test.ldd_parser.parsing_failed_reason
        out = output.print_results(output.results)
        assert 'W: unused-direct-shlib-dependency ' in out


@pytest.mark.skipif(not IS_X86_64, reason='x86-64 only')
def test_opt_dependency(binariescheck):
    output, test = binariescheck
    with FakePkg('fake') as pkg:
        pkgfile = pkg.add_file(get_full_path('opt-dependency'), '/bin/opt-dependency')
        run_elf_checks(test, pkg, pkgfile)
        assert not test.readelf_parser.parsing_failed_reason()
        assert not test.ldd_parser.parsing_failed_reason
        out = output.print_results(output.results)
        assert 'E: linked-against-opt-library /bin/opt-dependency /opt/libfoo.so' in out


@pytest.mark.skipif(not IS_X86_64, reason='x86-64 only')
def test_usr_dependency(binariescheck):
    output, test = binariescheck
    with FakePkg('fake') as pkg:
        pkgfile = pkg.add_file(get_full_path('usr-dependency'), '/bin/usr-dependency')
        run_elf_checks(test, pkg, pkgfile)
        assert not test.readelf_parser.parsing_failed_reason()
        assert not test.ldd_parser.parsing_failed_reason
        out = output.print_results(output.results)
        assert 'W: linked-against-usr-library /bin/usr-dependency /usr/libfoo.so' in out



from Testing import get_tested_mock_package


DbusRulePackage = get_tested_mock_package(
    lazyload=True,
    files={
        '/etc/dbus-1/system.d/noxml.conf': {},
        '/etc/dbus-1/system.d/org.freedesktop.NetworkManager.conf': {
            'content-path': 'files/systemd/org.freedesktop.NetworkManager.conf',
        },
        '/etc/dbus-1/system.d/org.freedesktop.NetworkManager2.conf': {
            'content-path': 'files/systemd/org.freedesktop.NetworkManager2.conf',
        },
    },
)



from mockdata.mock_dbus_policy import DbusRulePackage
import pytest
from rpmlint.checks.DBusPolicyCheck import DBusPolicyCheck
from rpmlint.filter import Filter

from Testing import CONFIG


@pytest.fixture(scope='function', autouse=True)
def dbuspolicycheck():
    CONFIG.info = True
    output = Filter(CONFIG)
    test = DBusPolicyCheck(CONFIG, output)
    return output, test


@pytest.mark.parametrize('package', [DbusRulePackage])
def test_dbus_policy(package, dbuspolicycheck):
    output, test = dbuspolicycheck
    test.check(package)
    out = output.print_results(output.results)
    assert 'E: dbus-parsing-exception raised an exception: no element found: line 1, column 0 /etc/dbus-1/system.d/noxml.conf' in out
    assert 'E: dbus-policy-allow-without-destination <allow send_interface="org.freedesktop.NetworkManager.PPP"/>' in out
    assert 'W: dbus-policy-allow-receive <allow receive_sender="foo"/>' in out
    assert 'E: dbus-policy-deny-without-destination <deny send_interface="org.freedesktop.NetworkManager.Settings" send_member="ReloadConnections"/>' in out
    assert 'E: dbus-policy-missing-allow /etc/dbus-1/system.d/org.freedesktop.NetworkManager2.conf' in out



import re

import pytest
from rpmlint.checks.SpecCheck import SpecCheck
from rpmlint.filter import Filter

from Testing import CONFIG, get_tested_package, get_tested_spec_package


@pytest.fixture(scope='function', autouse=True)
def speccheck():
    CONFIG.info = True
    output = Filter(CONFIG)
    test = SpecCheck(CONFIG, output)
    yield output, test


@pytest.fixture
def output(speccheck):
    output, _test = speccheck
    yield output


@pytest.fixture
def test(speccheck):
    _output, test = speccheck
    yield test


def test_check_include(tmp_path, speccheck):
    output, test = speccheck
    test.check_source(get_tested_package('source/CheckInclude', tmp_path))
    out = output.print_results(output.results)
    assert "specfile-error can't parse specfile" not in out
    assert 'no-buildroot-tag' in out
    assert 'E: specfile-error error: query of specfile' not in out


@pytest.mark.parametrize('package', [
    'spec/SpecCheck2',
    'spec/SpecCheck3',
    'spec/SpecCheckPatch',
    'spec/intltool',
    'spec/ghc',
])
def test_patch_not_applied(package, speccheck):
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'patch-not-applied' not in out


@pytest.mark.parametrize('package', ['spec/SpecCheck'])
def test_distribution_tags(package, speccheck):
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'patch-not-applied Patch3' in out
    assert not re.search(r'patch-not-applied Patch\b', out)
    assert not re.search('patch-not-applied Patch[0124567]', out)
    assert 'libdir-macro-in-noarch-package' not in out
    assert len(re.findall('macro-in-comment', out)) == 1
    assert 'unversioned-explicit-provides unversioned-provides' in out
    assert 'unversioned-explicit-provides versioned-provides' not in out
    assert 'unversioned-explicit-provides /' not in out
    assert 'unversioned-explicit-obsoletes unversioned-obsoletes' in out
    assert 'unversioned-explicit-obsoletes versioned-obsoletes' not in out
    assert 'unversioned-explicit-obsoletes /' not in out
    assert 'setup-not-quiet' in out


@pytest.mark.parametrize('package', ['spec/SpecCheck4'])
def test_forbidden_controlchars_found(package, speccheck):
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'E: forbidden-controlchar-found Requires:' in out
    assert 'E: forbidden-controlchar-found Provides:' in out
    assert 'E: forbidden-controlchar-found Obsoletes:' in out
    assert 'E: forbidden-controlchar-found Conflicts:' in out
    assert 'E: forbidden-controlchar-found %changelog:' in out


@pytest.mark.parametrize('package', ['source/no-spec-file'])
def test_check_no_spec_file(tmp_path, package, speccheck):
    """Test if spec file is not found inside RPM metadata."""
    output, test = speccheck
    test.check(get_tested_package(package, tmp_path))
    out = output.print_results(output.results)
    assert 'E: no-spec-file' in out


@pytest.mark.parametrize('package', ['source/CheckInclude'])
def test_check_no_spec_file_not_applied(tmp_path, package, speccheck):
    """Test if there is no spec file inside RPM metadata."""
    output, test = speccheck
    test.check(get_tested_package(package, tmp_path))
    out = output.print_results(output.results)
    assert 'E: no-spec-file' not in out


@pytest.mark.parametrize('package', ['spec/non-utf8-spec-file'])
def test_check_non_utf8_spec_file(package, speccheck):
    """Test if specfile does not have UTF-8 character encoding."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'E: non-utf8-spec-file' in out


@pytest.mark.parametrize('package', ['spec/SpecCheckTemp'])
def test_check_non_utf8_spec_file_not_applied(package, speccheck):
    """Test if specfile has UTF-8 character encoding."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'E: non-utf8-spec-file' not in out


@pytest.mark.parametrize('package', ['source/invalid-spec-name'])
def test_check_invalid_spec_name(tmp_path, package, speccheck):
    """Test if specfile name does not matches the ('Name: ') tag."""
    output, test = speccheck
    test.check(get_tested_package(package, tmp_path))
    out = output.print_results(output.results)
    assert 'E: invalid-spec-name' in out


@pytest.mark.parametrize('package', ['source/CheckInclude'])
def test_check_invalid_spec_name_not_applied(tmp_path, package, speccheck):
    """Test if specfile has specfile name as ('Name: ') tag."""
    output, test = speccheck
    test.check(get_tested_package(package, tmp_path))
    out = output.print_results(output.results)
    assert 'E: invalid-spec-name' not in out


@pytest.mark.parametrize('package', ['spec/non-break-space'])
def test_check_non_break_space(package, speccheck):
    """Test if specfile has a nbsp character."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: non-break-space' in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_non_break_space_not_applied(package, speccheck):
    """Test if specfile does not have any nbsp character."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: non-break-space' not in out


@pytest.mark.parametrize('package', ['spec/rpm-buildroot-usage'])
def test_check_rpm_buildroot_usage_under_prep(package, speccheck):
    """Test if specfile has buildroot macro under %prep."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'E: rpm-buildroot-usage' in out


@pytest.mark.parametrize('package', ['spec/SpecCheckTemp'])
def test_check_rpm_buildroot_usage_under_build(package, speccheck):
    """Test if specfile has buildroot macro under %build."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'E: rpm-buildroot-usage' in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_rpm_buildroot_usage_not_applied(package, speccheck):
    """Test if specfile does not have buildroot macro inside specfile."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'E: rpm-buildroot-usage' not in out


@pytest.mark.parametrize('package', ['spec/rpm-buildroot-usage-shell-var'])
def test_check_rpm_buildroot_usage_shell_var(package, speccheck):
    """Test detection of $RPM_BUILD_ROOT shell variable in %prep/%build"""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'E: rpm-buildroot-usage %prep' not in out
    assert out.count('E: rpm-buildroot-usage %build') == 2


@pytest.mark.parametrize('package', ['spec/make-check-outside-check-section'])
def test_check_make_check_outside_check_section(package, speccheck):
    """Test if specfile has `make check` outside %check."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: make-check-outside-check-section' in out


@pytest.mark.parametrize('package', ['spec/SpecCheckTemp'])
def test_check_make_check_inside_check_section(package, speccheck):
    """Test if specfile has `make check` inside all the required
    ('check', 'changelog', 'package', 'description') section.
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: make-check-outside-check-section' not in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_make_check_outside_not_applied(package, speccheck):
    """Test if specfile does not have any `make check`."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: make-check-outside-check-section' not in out


@pytest.mark.parametrize('package', ['spec/setup-not-quiet'])
def test_check_setup_not_quiet(package, speccheck):
    """Test if specfile does not have %setup -q macro."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: setup-not-quiet' in out


@pytest.mark.parametrize('package', ['spec/SpecCheckTemp'])
def test_check_setup_is_quiet(package, speccheck):
    """Test if specfile has a %setup -q macro."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: setup-not-quiet' not in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_setup_not_quiet_not_applied(package, speccheck):
    """Test if specfile does not have a setup macro."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: setup-not-quiet' not in out


@pytest.mark.parametrize('package', ['spec/setup-not-in-prep'])
def test_check_setup_not_in_prep(package, speccheck):
    """Test if specfile does not have %setup inside %prep."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: setup-not-in-prep' in out


@pytest.mark.parametrize('package', ['spec/SpecCheckTemp'])
def test_check_setup_inside_prep(package, speccheck):
    """Test if specfile has %setup inside %prep."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: setup-not-in-prep' not in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_setup_not_in_prep_not_applied(package, speccheck):
    """Test if specfile has no %setup."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: setup-not-in-prep' not in out


@pytest.mark.parametrize('package', ['spec/%autopatch-not-in-prep'])
def test_check_autopatch_not_in_prep(package, speccheck):
    """Test if specfile does not have %autopatch inside %prep."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: %autopatch-not-in-prep' in out


@pytest.mark.parametrize('package', ['spec/SpecCheckTemp'])
def test_check_autopatch_in_prep(package, speccheck):
    """Test if specfile has %autopatch inside %prep."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: %autopatch-not-in-prep' not in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_autopatch_not_in_prep_not_applied(package, speccheck):
    """Test if specfile has no %autopatch."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: %autopatch-not-in-prep' not in out


@pytest.mark.parametrize('package', ['spec/%autosetup-not-in-prep'])
def test_check_autosetup_not_in_prep(package, speccheck):
    """Test if specfile does not have %autosetup inside %prep."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: %autosetup-not-in-prep' in out


@pytest.mark.parametrize('package', ['spec/SpecCheckTemp'])
def test_check_autosetup_inside_prep(package, speccheck):
    """Test if specfile has %autosetup in %prep."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: %autosetup-not-in-prep' not in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_autosetup_not_in_prep_not_applied(package, speccheck):
    """Test if specfile has no %autosetup."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: %autosetup-not-in-prep' not in out


@pytest.mark.parametrize('package', ['spec/use-of-RPM-SOURCE-DIR'])
def test_check_use_of_rpm_source_dir(package, speccheck):
    """Test if specfile consist of $RPM_SOURCE_DIR or %{_sourcedir}."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'E: use-of-RPM_SOURCE_DIR' in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_use_of_rsd_not_applied(package, speccheck):
    """rsd: RPM_SOURCE_DIR
    Test if specfile does not consist of $RPM_SOURCE_DIR or %{_sourcedir}.
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'E: use-of-RPM_SOURCE_DIR' not in out


@pytest.mark.parametrize('package', ['spec/configure-without-libdir-spec'])
def test_check_configure_without_libdir_spec(package, speccheck):
    """Test if specfile does not have options augmented with --libdir."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: configure-without-libdir-spec' in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_cwls_not_applied(package, speccheck):
    """cwls: configure-without-libdir-spec
    Test if specfile has options augmented with --libdir.
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: configure-without-libdir-spec' not in out


@pytest.mark.parametrize('package', ['spec/hardcoded-library-path'])
def test_check_hardcoded_library_path(package, speccheck):
    """Test if specfile has hardcoded library path."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'E: hardcoded-library-path' in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_hclp_not_applied(package, speccheck):
    """hclp: hardcoded library path
    Test if specfile does not have hardcoded library path.
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'E: hardcoded-library-path' not in out


@pytest.mark.parametrize('package', ['spec/obsolete-tag'])
def test_check_obsolete_tag(package, speccheck):
    """Test if specfile has obsolete-tag as Copyright or Serial."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: obsolete-tag 2' in out
    assert 'W: obsolete-tag Something' in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_obsolete_tag_not_applied(package, speccheck):
    """Test if specfile does not have obsolete-tag as Copyright or Serial."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: obsolete-tag' not in out


@pytest.mark.parametrize('package', ['spec/hardcoded-path-in-buildroot-tag'])
def test_check_hardcoded_path_in_buildroot_tag(package, speccheck):
    """Test if specfile has hardoded path in buildroot tag."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: hardcoded-path-in-buildroot-tag /usr/bin/bash' in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_hpbt_not_applied(package, speccheck):
    """hpbt: hardcoded-path-in-buildroot-tag
    Test if specfile does not have hardoded path in buildroot tag.
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: hardcoded-path-in-buildroot-tag %{_tmppath}/%{name}-%{version}-%{release}-root-%(%{__id_u} -n)' not in out


@pytest.mark.parametrize('package', ['spec/buildarch-instead-of-exclusivearch-tag'])
def test_check_buildarch_instead_of_exclusivearch_tag(package, speccheck):
    """Test if specfile has BuildArch has any architecture beside noarch."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'E: buildarch-instead-of-exclusivearch-tag x86_64' in out
    assert 'E: buildarch-instead-of-exclusivearch-tag i586' in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_biet_not_applied(package, speccheck):
    """biet: buildarch-instead-of-exclusivearch-tag
    Test if specfile has BuildArch with noarch.
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'E: buildarch-instead-of-exclusivearch-tag noarch' not in out


@pytest.mark.parametrize('package', ['spec/hardcoded-packager-tag'])
def test_check_hardcoded_packager_tag(package, speccheck):
    """Test if specfile has hardcoded packager tag."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: hardcoded-packager-tag Someone' in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_hardcoded_packager_tag_not_applied(package, speccheck):
    """Test if specfile does not have hardcoded packager tag."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: hardcoded-packager-tag' not in out


@pytest.mark.parametrize('package', ['spec/hardcoded-prefix-tag'])
def test_check_hardcoded_prefix_tag(package, speccheck):
    """Test if specfile has hardcoded prefix tag."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: hardcoded-prefix-tag' in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_hardcoded_prefix_tag_not_applied(package, speccheck):
    """Test if specfile does not have hardcoded prefix tag."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: hardcoded-prefix-tag' not in out


@pytest.mark.parametrize('package', ['spec/prereq_use'])
def test_check_prereq_use(package, speccheck):
    """Test if specfile has tags such as PreReq(pre)
    or PreReq(post).
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'E: prereq-use none' in out
    assert 'E: prereq-use none_other' in out


@pytest.mark.parametrize('package', ['spec/patch-not-applied'])
def test_check_prereq_use_not_found(package, speccheck):
    """Test if specfile has no PreReq tag value."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'E: prereq-use' not in out


@pytest.mark.parametrize('package', ['spec/mixed-use-of-spaces-and-tabs'])
def test_check_prereq_use_not_applied(package, speccheck):
    """Test if specfile has no PreReq tag value."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'E: prereq-use' not in out


@pytest.mark.parametrize('package', ['spec/buildprereq-use'])
def test_check_buildprereq_use(package, speccheck):
    """Test if specfile has buildprereq tag."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'E: buildprereq-use Something' in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_buildprereq_use_not_applied(package, speccheck):
    """Test if specfile does not have buildprereq tag."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'E: buildprereq-use' not in out


@pytest.mark.parametrize('package', ['spec/forbidden-controlchar-found'])
def test_check_forbidden_controlchar_found(package, speccheck):
    """Test if specfile has forbidden controlchar
    in various parts of specfile.
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'E: forbidden-controlchar-found' in out


@pytest.mark.parametrize('package', ['spec/SpecCheckTemp'])
def test_check_forbidden_controlchar_not_found(package, speccheck):
    """Test if specfile does not have forbidden controlchar
    in various parts of specfile.
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'E: forbidden-controlchar-found' not in out


@pytest.mark.parametrize('package', ['spec/comparison-operator-in-deptoken'])
def test_check_coid(package, speccheck):
    """coid: comparison-operator-in-deptoken
    Test if specfile has comparison operator(>, =) in deptoken.
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: comparison-operator-in-deptoken something>2.0' in out
    assert 'W: comparison-operator-in-deptoken Something>1.0' in out
    assert 'W: comparison-operator-in-deptoken Something=2.0' in out


@pytest.mark.parametrize('package', ['spec/SpecCheckTemp'])
def test_check_coid_found(package, speccheck):
    """coid: comparison-operator-in-deptoken
    Test if specfile has comparison operator(<, <=) in deptoken.
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: comparison-operator-in-deptoken Someotherthing<1.0' in out
    assert 'W: comparison-operator-in-deptoken Someotherthing<=2.0' in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_coid_is_found(package, speccheck):
    """coid: comparison-operator-in-deptoken
    Test if specfile has comparison operator(==, >=) in deptoken.
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: comparison-operator-in-deptoken Oneanotherthing>=1.0' in out
    assert 'W: comparison-operator-in-deptoken Onelastthing==2.0' in out
    assert 'W: comparison-operator-in-deptoken source-for-second-rpm' not in out


@pytest.mark.parametrize('package', ['spec/%autopatch-not-in-prep'])
def test_check_coid_is_found_with_single_space(package, speccheck):
    """coid: comparison-operator-in-deptoken
    Test if specfile has comparison operator(>=, <=) with single space in deptoken.
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: comparison-operator-in-deptoken Someotherthinwithsinglespace<=' in out


@pytest.mark.parametrize('package', ['spec/%autosetup-not-in-prep'])
def test_check_coid_is_found_with_double_space(package, speccheck):
    """coid: comparison-operator-in-deptoken
    Test if specfile has comparison operator(>=, <=) with single space in deptoken.
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: comparison-operator-in-deptoken /Something' not in out


@pytest.mark.parametrize('package', ['spec/unversioned-explicit-version'])
def test_check_unversioned_explicit_version(package, speccheck):
    """Test if specfile has Provides: tag."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: unversioned-explicit-provides someones-something=%{version}' in out


@pytest.mark.parametrize('package', ['spec/%autosetup-not-in-prep'])
def test_check_unversioned_explicit_version_not_found(package, speccheck):
    """Test if specfile has Provides: /something tag."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: unversioned-explicit-provides /something' not in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_uev_not_applied(package, speccheck):
    """uev: unversioned-explicit-version
    1. Test if specfile does not have Provides: tag.
    2. Test if specfile does not have Obsoletes: tag.
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: unversioned-explicit-provides' not in out


@pytest.mark.parametrize('package', ['spec/unversioned-explicit-obsoletes'])
def test_check_unversioned_explicit_obsoletes(package, speccheck):
    """Test if specfile has Obsoletes: tag."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: unversioned-explicit-obsoletes Something' in out


@pytest.mark.parametrize('package', ['spec/%autopatch-not-in-prep'])
def test_check_unversioned_explicit_obsoletes_not_found(package, speccheck):
    """Test if specfile has correct Obsoletes: tag
    and Provides: tag in specfile.
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: unversioned-explicit-obsoletes %{name} <= %{version}' not in out
    assert 'W: unversioned-explicit-obsoletes %{name} = %{version}' not in out


@pytest.mark.parametrize('package', ['spec/SpecCheckTemp'])
def test_check_ueo_not_found(package, speccheck):
    """ueo: unversioned-explicit-obsoletes
    Test if specfile does has Obsoletes: /something.
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: unversioned-explicit-obsoletes /something' not in out


@pytest.mark.parametrize('package', ['spec/macro-in-changelog'])
def test_check_macro_in_changelog(package, speccheck):
    """Test if specfile has macro in %changelog."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: macro-in-%changelog' in out


@pytest.mark.parametrize('package', ['spec/macro-in-changelog-autochangelog'])
def test_check_autochangelog(package, speccheck):
    """Test usage of %autochangelog macro."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: macro-in-%changelog' not in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_macro_in_changelog_not_found(package, speccheck):
    """Test if specfile has macro in %changelog
    consisting of %%foo or %+foo or %.foo.
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: macro-in-%changelog' not in out


@pytest.mark.parametrize('package', ['spec/SpecCheckTemp'])
def test_check_macro_in_changelog_not_applied(package, speccheck):
    """Test if specfile does not have macro in %changelog."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: macro-in-%changelog' not in out


@pytest.mark.parametrize('package', ['spec/libdir-macro-in-noarch-package'])
def test_check_libdir_macro_in_noarch_package(package, speccheck):
    """Test if specfile has _libdir macro in noarch package."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: libdir-macro-in-noarch-package' in out


@pytest.mark.parametrize('package', ['spec/mixed-use-of-spaces-and-tabs'])
def test_check_lmnp_not_applied(package, speccheck):
    """lmnp: libdir-macro-in-noarch-package
    Test if specfile does not have _libdir macro in noarch package.
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: libdir-macro-in-noarch-package' not in out


@pytest.mark.parametrize('package', ['spec/deprecated-grep'])
def test_check_deprecated_grep(package, speccheck):
    """Test if specfile has direct use of grep or egrep or fgrep."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: deprecated-grep' in out


@pytest.mark.parametrize('package', ['spec/SpecCheckTemp'])
def test_check_deprecated_grep_not_found(package, speccheck):
    """Test if specfile has grep with -F or -E inside
    package, changelog, depscription, files macro
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: deprecated-grep' not in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_deprecated_grep_not_applied(package, speccheck):
    """Test if specfile has no use of egrep/fgrep or egrep/fgrep with -E or -F."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: deprecated-grep' not in out


# TODO: Add test for non-standard-Group


@pytest.mark.parametrize('package', ['spec/macro-in-comment'])
def test_check_macro_in_comment(package, speccheck):
    """Test if specfile has macro in comment."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: macro-in-comment' in out


@pytest.mark.parametrize('package', ['spec/%autosetup-not-in-prep'])
def test_check_macro_in_comment_not_found(package, speccheck):
    """Test if specfile has comment in macro."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: macro-in-comment' not in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_macro_in_comment_not_applied(package, speccheck):
    """Test if specfile does not have macro inside a comment."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: macro-in-comment' not in out


@pytest.mark.parametrize('package', ['spec/no-buildroot-tag'])
def test_check_no_build_root_tag(package, speccheck):
    """Test if specfile does not have BuildRoot tag."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: no-buildroot-tag' in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_no_build_root_tag_not_applied(package, speccheck):
    """Test if specfile has BuildRoot tag."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: no-buildroot-tag' not in out


@pytest.mark.parametrize('package', ['spec/no-%%%s-section'])
def test_check_no_essential_section(package, speccheck):
    """Test for no-%%%s-section check
    Test if specfile does not have essential section tag.
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: no-%prep-section' in out
    assert 'W: no-%install-section' in out
    assert 'W: no-%build-section' in out
    assert 'W: no-%check-section' in out


@pytest.mark.parametrize('package', ['spec/libspelling'])
def test_check_no_essential_section_declarative(package, speccheck):
    """Test for declarative build check
    Test if specfile does not have essential section tag but uses declarative
    build.
    https://github.com/rpm-software-management/rpmlint/issues/1311
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: no-%prep-section' not in out
    assert 'W: no-%install-section' not in out
    assert 'W: no-%build-section' not in out
    assert 'W: no-%check-section' not in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_no_essential_section_not_applied(package, speccheck):
    """Test for no-%%%s-section check
    Test if specfile has all essential section tag.
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: no-%prep-section' not in out
    assert 'W: no-%install-section' not in out
    assert 'E: superfluous-%clean-section' in out
    assert 'W: no-%build-section' not in out
    assert 'W: no-%check-section' not in out


@pytest.mark.parametrize('package', ['spec/more-than-one-%changelog'])
def test_check_more_than_one_changelog_section(package, speccheck):
    """Test if specfile has more than one changelog section."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: more-than-one-%changelog-section' in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_mtocs_not_applied(package, speccheck):
    """mtocs: more-than-one-%changelog-section
    Test if specfile does not have more than one changelog section.
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: more-than-one-%changelog-section' not in out


@pytest.mark.parametrize('package', ['spec/lib-package-without-%mklibname'])
def test_check_lib_package_without_mklibname(package, speccheck):
    """Test if specfile has lib pacakge without %mklibname."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'E: lib-package-without-%mklibname' in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_lpwm_not_applied(package, speccheck):
    """lpwm: lib-package-without-%mklibname
    Test if specfile does not have lib pacakge without %mklibname."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'E: lib-package-without-%mklibname' not in out


@pytest.mark.parametrize('package', ['spec/depscript-without-disabling-depgen'])
def test_check_depscript_without_disabling_depgen(package, speccheck):
    """Test if specfile has define __find_provides/requires."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: depscript-without-disabling-depgen' in out


@pytest.mark.parametrize('package', ['spec/SpecCheckTemp'])
def test_check_depscript_without_disabling_depgen_not_found(package, speccheck):
    """Test if specfile has define __find_provides/requires with
    %define _use_internal_dependency_generator set to 0.
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: depscript-without-disabling-depgen' not in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_dwdd_not_applied(package, speccheck):
    """dwdd: depscript-without-disabling-depgen
    Test if specfile does not have define __find_provides/requires."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: depscript-without-disabling-depgen' not in out


@pytest.mark.parametrize('package', ['spec/patch-fuzz-is-changed'])
def test_check_patch_fuzz_is_changed(package, speccheck):
    """Test if specfile has internal/default patch fuzz value changed as
    %define _default_patch_fuzz >= 0.
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: patch-fuzz-is-changed' in out


@pytest.mark.parametrize('package', ['spec/SpecCheckTemp'])
def test_check_patch_fuzz_is_changed_not_found(package, speccheck):
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: patch-fuzz-is-changed' not in out


@pytest.mark.parametrize('package', ['spec/macro-in-comment'])
def test_check_patch_fuzz_is_changed_not_applied(package, speccheck):
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: patch-fuzz-is-changed' not in out


@pytest.mark.parametrize('package', ['spec/mixed-use-of-spaces-and-tabs'])
def test_check_mixed_use_of_spaces_and_tabs(package, speccheck):
    """Test if specfile has mix use of space and tabs."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: mixed-use-of-spaces-and-tabs' in out


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_mixed_use_of_spaces_and_tabs_not_applied(package, speccheck):
    """Test if specfile does not have mix use of space and tabs."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: mixed-use-of-spaces-and-tabs' not in out


@pytest.mark.parametrize('package', ['spec/%ifarch-applied-patch'])
def test_check_ifarch_applied_patch(package, speccheck):
    """Test if specfile has no patch in %ifarch block."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: %ifarch-applied-patch' in out


@pytest.mark.parametrize('package', ['spec/mixed-use-of-spaces-and-tabs'])
def test_check_ifarch_applied_not_enforced(package, speccheck):
    """Test if specfile has patch in %ifarch block."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: %ifarch-applied-patch' not in out


@pytest.mark.parametrize('package', ['spec/patch-not-applied'])
def test_check_patch_not_applied(package, speccheck):
    """Test if specfile does not have all patch applied."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: patch-not-applied' in out


@pytest.mark.parametrize('package', [
    'spec/SpecCheck',
    'spec/%ifarch-applied-patch',
    'spec/prereq_use',
    'spec/mixed-use-of-spaces-and-tabs',
])
def test_check_patch_rpm_420(package, speccheck):
    """Test if specfile uses %patchN."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'E: patch-macro-old-format' in out


@pytest.mark.parametrize('package', [
    'spec/intltool',
    'spec/ghc',
    'spec/SpecCheckPatch',
])
def test_check_patch_rpm_not_420(package, speccheck):
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'E: patch-macro-old-format' not in out


@pytest.mark.parametrize('package', ['spec/mixed-use-of-spaces-and-tabs'])
def test_check_patch_not_found(package, speccheck):
    """Test if specfile have all patch applied by %autopatch."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: patch-not-applied' not in out


@pytest.mark.parametrize('package', ['spec/mixed-use-of-spaces-and-tabs'])
def test_check_patch_not_applied_not_enforced(package, speccheck):
    """Test if specfile has all patch applied."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: patch-not-applied' not in out


# TODO: Add specfile-error test.


@pytest.mark.parametrize('package', ['spec/SpecCheck2'])
def test_check_invalid_url(package, speccheck):
    """Test if specfile has invalid url."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: invalid-url' in out


@pytest.mark.parametrize('package', ['spec/mixed-use-of-spaces-and-tabs'])
def test_check_invalid_url_not_applied(package, speccheck):
    """Test if specfile does not have invalid url."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: invalid-url' not in out


@pytest.mark.parametrize('package', ['spec/python-setup-test'])
def test_python_setup_test(package, speccheck):
    """Test if specfile has deprecated use of 'setup.py test'."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: python-setup-test' in out


@pytest.mark.parametrize('package', ['spec/python-module-def'])
def test_python_module_definition(package, speccheck):
    """Test if python_module macro is defined in the spec file."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: python-module-def' in out


@pytest.mark.parametrize('package', [
    'spec/SpecCheck',
    'spec/SpecCheck2',
    'spec/SpecCheck3',
    'spec/SpecCheck4',
])
def test_python_module_definition_not_present(package, speccheck):
    """Test if python_module macro warning is not shown if the macro is not
       defined in the spec file.
    """
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: python-module-def' not in out


@pytest.mark.parametrize('package', [
    'spec/python-sitelib-glob',
    'spec/python-sitearch-glob',
])
def test_python_sitelib_glob(package, speccheck):
    """Test if %{python_sitelib}/* is present in %files section."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: python-sitelib-glob-in-files' in out


@pytest.mark.parametrize('package', ['spec/python-sitelib'])
def test_python_sitelib(package, speccheck):
    """Test if %{python_sitelib}/* is present in %files section."""
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'W: python-sitelib-glob-in-files' not in out


@pytest.mark.parametrize('package', [
    'spec/null-char-last',
    'spec/null-char-first',
])
def test_null_char(package, speccheck):
    output, test = speccheck
    pkg = get_tested_spec_package(package)
    test.check_spec(pkg)
    out = output.print_results(output.results)
    assert 'forbidden-controlchar-found' in out


@pytest.mark.parametrize('package', [
    get_tested_spec_package('spec/MacroInComment'),
])
def test_special_comments(package, output, test):
    test.check_spec(package)
    out = output.print_results(output.results)
    assert 'W: macro-in-comment %configure' in out
    assert 'W: macro-in-comment %{name}' not in out


@pytest.mark.parametrize('spec,expected', [
    ('spec/gimp', True),
    ('spec/libcanberra', True),
    ('spec/MacroInComment', False),
    ('spec/ghc', False),
    ('spec/yast2-installation', False),
])
def test_deprecated_suse_update_desktop_files(spec, expected, output, test):
    package = get_tested_spec_package(spec)
    test.check_spec(package)
    out = output.print_results(output.results)
    assert ('W: suse-update-desktop-file-deprecated' in out) == expected



from mockdata.mock_i18n import I18NPackage, InvalidI18NPackage
import pytest
from rpmlint.checks.I18NCheck import I18NCheck
from rpmlint.filter import Filter

from Testing import CONFIG


@pytest.fixture(scope='function', autouse=True)
def i18ncheck():
    CONFIG.info = True
    output = Filter(CONFIG)
    test = I18NCheck(CONFIG, output)
    yield output, test


@pytest.fixture
def output(i18ncheck):
    output, _test = i18ncheck
    yield output


@pytest.fixture
def test(i18ncheck):
    _output, test = i18ncheck
    yield test


@pytest.mark.parametrize('package', [InvalidI18NPackage])
def test_i18n_invalid_lang(package, output, test):
    test.check(package)
    out = output.print_results(output.results)
    assert 'E: invalid-lc-messages-dir /usr/share/locale/xx_ES/LC_MESSAGES/goodvibes.mo' in out
    assert 'E: invalid-lc-messages-dir /usr/share/locale/es_XX/LC_MESSAGES/goodvibes.mo' in out
    assert 'E: invalid-lc-messages-dir /usr/share/locale/xx/LC_MESSAGES/goodvibes.mo' in out


@pytest.mark.parametrize('package', [I18NPackage])
def test_i18n_valid_lang(package, output, test):
    test.check(package)
    out = output.print_results(output.results)
    assert 'E: invalid-lc-messages-dir' not in out



import contextlib
from itertools import chain
import pathlib
import sys
import tempfile

import rpm
from rpmlint.helpers import byte_to_string, print_warning
from rpmlint.pkg import get_installed_pkgs, Pkg


class Rpmdiff:
    # constants
    TAGS = (rpm.RPMTAG_NAME, rpm.RPMTAG_SUMMARY,
            rpm.RPMTAG_DESCRIPTION, rpm.RPMTAG_GROUP,
            rpm.RPMTAG_LICENSE, rpm.RPMTAG_URL,
            rpm.RPMTAG_PREIN, rpm.RPMTAG_POSTIN,
            rpm.RPMTAG_PREUN, rpm.RPMTAG_POSTUN,
            rpm.RPMTAG_PRETRANS, rpm.RPMTAG_POSTTRANS)

    PRCO = ('REQUIRES', 'PROVIDES', 'CONFLICTS', 'OBSOLETES',
            'RECOMMENDS', 'SUGGESTS', 'ENHANCES', 'SUPPLEMENTS')

    __FILEIDX = [['S', 'size'],
                 ['M', 'mode'],
                 ['5', 'digest'],
                 ['D', 'rdev'],
                 ['N', 'nlink'],
                 ['L', 'state'],
                 ['V', 'vflags'],
                 ['U', 'user'],
                 ['G', 'group'],
                 ['F', 'fflags'],
                 ['T', 'mtime']]

    DEPFORMAT = '%-12s%s %s %s %s'
    FORMAT = '%-12s%s'

    ADDED = 'added'
    REMOVED = 'removed'

    def __init__(self, old, new, ignore=None, exclude=None):
        self.result = []
        self.ignore = ignore or []
        self.exclude = exclude or []

        FILEIDX = self.__FILEIDX
        for tag in self.ignore:
            for entry in FILEIDX:
                if tag == entry[0]:
                    entry[1] = None
                    break

        try:
            old = self.__load_pkg(old).header
            new = self.__load_pkg(new).header
        except KeyError as e:
            print_warning(str(e))
            sys.exit(2)

        # Compare single tags
        for tag in self.TAGS:
            old_tag = old[tag]
            new_tag = new[tag]
            if old_tag != new_tag:
                tagname = rpm.tagnames[tag]
                if old_tag is None:
                    self.__add(self.FORMAT, (self.ADDED, tagname))
                elif new_tag is None:
                    self.__add(self.FORMAT, (self.REMOVED, tagname))
                else:
                    self.__add(self.FORMAT, ('S.5.....', tagname))

        # compare Provides, Requires, ...
        for tag in self.PRCO:
            self.__comparePRCOs(old, new, tag)

        # compare the files
        old_files_dict = self.__fileIteratorToDict(rpm.files(old))
        new_files_dict = self.__fileIteratorToDict(rpm.files(new))
        files = list(set(chain(iter(old_files_dict), iter(new_files_dict))))
        files.sort()

        for f in files:
            if self._excluded(f):
                continue

            diff = False

            old_file = old_files_dict.get(f)
            new_file = new_files_dict.get(f)

            if not old_file:
                self.__add(self.FORMAT, (self.ADDED, f))
            elif not new_file:
                self.__add(self.FORMAT, (self.REMOVED, f))
            else:
                fmt = ''
                for entry in FILEIDX:
                    if entry[1] is not None and \
                            getattr(old_file, entry[1]) != getattr(new_file, entry[1]):
                        fmt += entry[0]
                        diff = True
                    else:
                        fmt += '.'
                if diff:
                    self.__add(self.FORMAT, (fmt, f))

    def _excluded(self, f):
        f = pathlib.PurePath(f)
        for glob in self.exclude:
            if f.match(glob):
                return True
            if glob.startswith('/'):
                for parent in f.parents:
                    if parent.match(glob):
                        return True
        return False

    # return a report of the differences
    def textdiff(self):
        return '\n'.join((fmt % data for fmt, data in self.result))

    # do the two rpms differ
    def differs(self):
        return bool(self.result)

    # add one differing item
    def __add(self, fmt, data):
        self.result.append((fmt, data))

    # load a package from a file or from the installed ones
    def __load_pkg(self, name):
        # FIXME: redo to try file/installed and proceed based on that, or pick
        # one of the selected first
        tmpdir = tempfile.gettempdir()
        with contextlib.suppress(TypeError):
            if name.is_file():
                return Pkg(name, tmpdir)
        inst = get_installed_pkgs(str(name))
        if not inst:
            raise KeyError(f'No installed packages by name {name}')
        if len(inst) > 1:
            raise KeyError(f'More than one installed packages by name {name}')
        return inst[0]

    # output the right string according to RPMSENSE_* const
    def sense2str(self, sense):
        s = ''
        for tag, char in ((rpm.RPMSENSE_LESS, '<'),
                          (rpm.RPMSENSE_GREATER, '>'),
                          (rpm.RPMSENSE_EQUAL, '=')):
            if sense & tag:
                s += char
        return s

    # output the right requires string according to RPMSENSE_* const
    def req2str(self, req):
        s = 'REQUIRES'
        # we want to use 64 even with rpm versions that define RPMSENSE_PREREQ
        # as 0 to get sane results when comparing packages built with an old
        # (64) version and a new (0) one
        if req & (rpm.RPMSENSE_PREREQ or 64):
            s = 'PREREQ'

        ss = []
        if req & rpm.RPMSENSE_SCRIPT_PRE:
            ss.append('pre')
        if req & rpm.RPMSENSE_SCRIPT_POST:
            ss.append('post')
        if req & rpm.RPMSENSE_SCRIPT_PREUN:
            ss.append('preun')
        if req & rpm.RPMSENSE_SCRIPT_POSTUN:
            ss.append('postun')
        if req & getattr(rpm, 'RPMSENSE_PRETRANS', 1 << 7):  # rpm >= 4.9.0
            ss.append('pretrans')
        if req & getattr(rpm, 'RPMSENSE_POSTTRANS', 1 << 5):  # rpm >= 4.9.0
            ss.append('posttrans')
        if ss:
            s += '(%s)' % ','.join(ss)

        return s

    # compare Provides, Requires, Conflicts, Obsoletes
    def __comparePRCOs(self, old, new, name):
        try:
            oldflags = old[name[:-1] + 'FLAGS']
        except ValueError:
            # assume tag not supported, e.g. Recommends with older rpm
            return
        newflags = new[name[:-1] + 'FLAGS']
        # fix buggy rpm binding not returning list for single entries
        if not isinstance(oldflags, list):
            oldflags = [oldflags]
        if not isinstance(newflags, list):
            newflags = [newflags]

        o = zip(old[name], oldflags, old[name[:-1] + 'VERSION'])
        if not isinstance(o, list):
            o = list(o)
        n = zip(new[name], newflags, new[name[:-1] + 'VERSION'])
        if not isinstance(n, list):
            n = list(n)

        # filter self provides, TODO: self %name(%_isa) as well
        if name == 'PROVIDES':
            oldE = old['epoch'] is not None and str(old['epoch']) + ':' or ''
            oldV = '{}{}'.format(oldE, old.format('%{VERSION}-%{RELEASE}'))
            oldNV = (old['name'], rpm.RPMSENSE_EQUAL, oldV.encode())
            newE = new['epoch'] is not None and str(new['epoch']) + ':' or ''
            newV = '{}{}'.format(newE, new.format('%{VERSION}-%{RELEASE}'))
            newNV = (new['name'], rpm.RPMSENSE_EQUAL, newV.encode())
            o = [entry for entry in o if entry != oldNV]
            n = [entry for entry in n if entry != newNV]

        for oldentry in o:
            if oldentry not in n:
                namestr = name
                if namestr == 'REQUIRES':
                    namestr = self.req2str(oldentry[1])
                self.__add(self.DEPFORMAT,
                           (self.REMOVED, namestr, byte_to_string(oldentry[0]),
                            self.sense2str(oldentry[1]), byte_to_string(oldentry[2])))
        for newentry in n:
            if newentry not in o:
                namestr = name
                if namestr == 'REQUIRES':
                    namestr = self.req2str(newentry[1])
                self.__add(self.DEPFORMAT,
                           (self.ADDED, namestr, byte_to_string(newentry[0]),
                            self.sense2str(newentry[1]), byte_to_string(newentry[2])))

    def __fileIteratorToDict(self, fi):
        result = {}
        for filedata in fi:
            result[filedata.name] = filedata
        return result



# File containing various helper functions used across rpmlint

from contextlib import contextmanager
import os
from shutil import get_terminal_size
import sys

from rpmlint.color import Color


ENGLISH_ENVIRONMENT = dict(os.environ, LC_ALL='en_US.UTF-8', LANGUAGE='en_US')


def string_center(message, filler=' '):
    """
    Create string centered of the terminal
    """
    cols, rows = get_terminal_size()
    return (f' {message} ').center(cols, filler)


def print_centered(message, filler=' '):
    """
    Print message in the center of a terminal
    """
    print(string_center(message, filler))


def print_warning(message):
    """
    Print warning message to stderr.
    """
    print(f'{Color.Red}{message}{Color.Reset}', file=sys.stderr)


def byte_to_string(item):
    """
    Convert byte items to strings
    """
    # empty stuff or already existing string stays
    if item is None or isinstance(item, str):
        return item

    # if we have a list/tuple we have to recurse
    if isinstance(item, (list, tuple)):
        return [byte_to_string(i) for i in item]

    # everything else shall be decoded and fails replaced
    return item.decode(encoding='UTF-8', errors='replace')


def readlines(path):
    with open(path, 'rb') as fobj:
        for line in fobj:
            yield byte_to_string(line)


@contextmanager
def pushd(new_dir):
    """
    Mimics Unix pushd/popd
    """
    cwd = os.getcwd()
    os.chdir(new_dir)
    try:
        yield
    finally:
        os.chdir(cwd)



from Testing import get_tested_mock_package


AppDataPackage = get_tested_mock_package(
    lazyload=True,
    files={
        '/usr/share/appdata/broken.appdata.xml': {
            'content-path': 'files/broken.appdata.xml',
            'create_dirs': True,
        },
        '/usr/share/appdata/broken-xml.metainfo.xml': {
            'content-path': 'files/broken-xml.metainfo.xml',
            'create_dirs': True,
        },
    }
)



import re

from rpmlint.checks.AbstractCheck import AbstractCheck


class PAMModulesCheck(AbstractCheck):
    pam_module_re = re.compile(r'^(?:/usr)?/lib(?:64)?/security/([^/]+\.so)$')

    def __init__(self, config, output):
        super().__init__(config, output)
        self.pam_authorized_modules = config.configuration['PAMAuthorizedModules']

    def check(self, pkg):
        if pkg.is_source:
            return

        for f in pkg.files:
            if f in pkg.ghost_files:
                continue

            m = self.pam_module_re.match(f)
            if m:
                bn = m.groups()[0]
                if bn not in self.pam_authorized_modules:
                    self.output.add_info('E', pkg, 'pam-unauthorized-module', bn)



import re
import stat
import time

import rpm
from rpmlint.checks.AbstractCheck import AbstractFilesCheck


class BuildRootAndDateCheck(AbstractFilesCheck):
    """
    Check that the file doesn't contain the current date or time.
    And check the file does not contain build root reference.

    If so, it causes the package to rebuild when it's not needed.
    """
    def __init__(self, config, output):
        super().__init__(config, output, r'.*')
        self.looksliketime = re.compile('(2[0-3]|[01]?[0-9]):([0-5]?[0-9]):([0-5]?[0-9])')
        self.istoday = re.compile(time.strftime('%b %e %Y'))
        self.prepare_regex(rpm.expandMacro('%{?buildroot}') or '^/.*/BUILDROOT/')

    def prepare_regex(self, buildroot):
        for m in ('name', 'version', 'release', 'NAME', 'VERSION', 'RELEASE'):
            buildroot = buildroot.replace('%%{%s}' % (m), r'[\w\!-\.]{1,20}')
        self.build_root_re = re.compile(buildroot)

    def check_file(self, pkg, filename):
        if filename.startswith('/usr/lib/debug') or pkg.is_source or \
                not stat.S_ISREG(pkg.files[filename].mode):
            return

        data = pkg.read_with_mmap(filename)
        if self.istoday.search(data):
            if self.looksliketime.search(data):
                self.output.add_info('E', pkg, 'file-contains-date-and-time', filename)
            else:
                self.output.add_info('E', pkg, 'file-contains-current-date', filename)
        if self.build_root_re.search(data):
            self.output.add_info('E', pkg, 'file-contains-buildroot', filename)



from Testing import get_tested_mock_package


MixedOwnershipPackage = get_tested_mock_package(
    lazyload=True,
    files={
        '/usr/bin/noproblem': {},
        '/var/lib/badfolder': {
            'metadata': {'user': 'nobody'},
            'is_dir': True
        },
        '/var/lib/badfolder/broken1': {
            'metadata': {'user': 'root'}
        },
        '/var/lib/badfolder/correctperms': {
            'metadata': {'user': 'root'},
            'is_dir': True,
        },
        '/var/lib/badfolder/correctperms/broken2': {},
    },
)



from rpmlint.checks.AbstractCheck import AbstractCheck


class MixedOwnershipCheck(AbstractCheck):
    def check(self, pkg):
        """
        Check for mixed permissions in the directory path.

        If folder is owned by i.e. nobody then the files there shouldn't
        be owned by other user either, as then nobody could replace the files
        and inject anything.
        """
        if pkg.is_source:
            return

        for path, info in pkg.files.items():
            parent = path.rpartition('/')[0]

            # In case parent folder is not part of this RPM we can't verify it
            if parent not in pkg.files:
                continue
            parent_owner = pkg.files[parent].user
            # root user is trusted
            if parent_owner in ('root', '0'):
                continue
            if info.user != parent_owner:
                message = f'Path "{path}" owned by "{info.user}" is stored in directory owned by "{parent_owner}"'
                self.output.add_info('E', pkg, 'file-parent-ownership-mismatch', message)
